
package softparsmap;


import ooc.*;
import java.util.*;

/**
 * Extending this class will facilitate creating a tag instance
 * that is needed for any class to be installed and used by the
 * property file. 
 */
public abstract class AbstractInparalogous extends LinkedTagInstance
    implements Inparalogous {

    public boolean allLeavesIsFromSameSpecies(GeneNode node) {
	return node.getSL().size()==1;
    }
    
    public Set getInparalogous(Family family, GeneNode node) {
	Set inp = new HashSet();
	for(Iterator i = node.getLeaves().iterator();i.hasNext();) {
	    GeneNode n = (GeneNode)i.next();
	    boolean foundInp = false;
	    while(!n.isRoot() &&
		  allLeavesIsFromSameSpecies((GeneNode)n.getParent())){
		n = (GeneNode)n.getParent();
		foundInp = true;
	    }
	    if(foundInp) {
		inp.add(n);
	    }
	}
	return inp;
    }

    public boolean removeWhileMinimizingMutation() {
	return getAttributeBoolean("remove_while_minimizing_mutation");
    }

    public boolean removeBeforeSaving() {
	return getAttributeBoolean("remove_before_saving");
    }

    public void setInparalogousOrder(GeneNode root) {
	// setting inparalogous order
	Set leaves = root.getLeaves();
	if(((GeneLeaf)leaves.iterator().next()).getInparalogousOrder()<0) {
	    Object[] leavesArray = leaves.toArray();
	    Arrays.sort(leavesArray, getLeafComparator(root.getFamily()));
	    int k=1;
	    for(int i=0;i<leavesArray.length;i++) {
		((GeneLeaf)leavesArray[i]).setInparalogousOrder(k);
		k++;
	    }
	    //System.out.println("******************* \n"+root.toStringAll());
	}
    }
    
    public GeneNode removeInparalogous(Family family, GeneNode node) {
	if(!node.isRoot()) {
	    throw new FatalException("Can only be done on the root.");
	}
	GeneNode root = node;
	// checking inparalogous order
	setInparalogousOrder(root);
	// get and remove inparalogous
	Set inpara = getInparalogous(family, node);
	for(Iterator i = inpara.iterator();i.hasNext();) {
	    GeneNode n = (GeneNode)i.next();
	    TreeSet s = new TreeSet(getOrderComparator());
	    s.addAll(n.getLeaves());
	    GeneLeaf l = (GeneLeaf)s.first();
	    if(!n.isRoot()) {
		Node p = n.getParent();
		p.removeChild(n);
		p.addChild(l);		
	    } else {
		if(!l.isRoot()) {
		    l.getParent().removeChild(l);
		}
		return l;
	    }
	}
	return root;
    }

    private Comparator getOrderComparator() {
	return new Comparator() {
		public int compare(Object o1, Object o2) {
		    int io1 = ((GeneLeaf)o1).getInparalogousOrder();
		    int io2 = ((GeneLeaf)o2).getInparalogousOrder();
		    if(io1<0 || io2<0) {
			throw new FatalException
			    ("Inparalogous order is not set.");
		    }
		    return io1-io2;
		}
	    };
    }

}
