
package softparsmap;


import ooc.*;
import java.io.*;

/**
 * If the sequences are stored in one file this class can be extended
 * and used together with the class DataSourceXmlGenData.
 */
public abstract class AbstractSequenceData extends LinkedTagInstance
    implements SequenceData {

    /**
     * Returns the property file.
     *
     * @return the property file
     * @see PropertyFile
     */
    public PropertyFile getPropertyFile() {
	return (PropertyFile)getTag().getContainer();
    }

    /**
     * Returns a new {@link FilePointerPart} on current position in
     * the file.
     *
     * @return a {@link FilePointerPart} on current position in file
     */
    public abstract FilePointerPart newFilePointerPartCurrent();

    /**
     * Sets the file that contains the sequences.
     *
     * @param raf is the file containing the sequences
     */
    public abstract void setDatabaseFile(RandomAccessFile raf);

    /**
     * Move to next sequence in file.
     *
     * @return true if there is a next sequence and the current pointer
     *         is set to this sequence
     */
    public abstract boolean moveNextSequence();

    /**
     * Parse in a sequence given a the file and pointer.
     *
     * @param raf is the file containing all the sequences
     * @param fpp is the file pointer
     */
    public abstract void parseSequence(RandomAccessFile raf,
				       FilePointerPart fpp);
   
}
