    
package softparsmap;


import ooc.*;
import java.io.*;

/**
 * Extending this class will facilitate creating a tag instance
 * that is needed for any class to be installed and used by the
 * property file.
 */
public abstract class AbstractTreeParser extends LinkedTagInstance
    implements TreeParser {

    /**
     * Returns the property file.
     *
     * @return the property file
     * @see PropertyFile
     */    
    public PropertyFile getPropertyFile() {
	return (PropertyFile)getTag().getContainer();
    }

    /**
     * Returns the name of the encoding. Defined in attribute 'encoding'.
     *
     * @return the name of the encoding
     */
    public String getCharset() {
	return getAttributeString("encoding");
    }
    
    public Node parseIn(InputStream tree) {
	return parseIn(convert(tree));
    }

    public void parseIn(Node tree, InputStream treeInfo) {
	parseIn(tree, convert(treeInfo));
    }

    private String convert(InputStream tree) {
	try {
	    // convert the stream into a string
	    BufferedReader br =
		new BufferedReader(new InputStreamReader(tree, getCharset()));
	    StringBuffer sb = new StringBuffer();
	    String text = null;
	    do {
		text = br.readLine();
		if(text!=null) {
		    sb.append(text);
		}
	    } while(text!=null);
	    // convert string and return tree	    
	    return sb.toString();
	} catch(Exception e) {
	    throw new FatalException(e);
	}
    }

    public void parseOut(OutputStream outputStream, Node tree) {
	try {
	    OutputStreamWriter osw = new OutputStreamWriter
		(outputStream, getCharset());
	    osw.write(parseOut(tree));
	    osw.close();
	} catch(Exception e) {
	    throw new FatalException(e);
	}
    }

    public abstract Node parseIn(String tree);

    public abstract void parseIn(Node tree, String treeInfo);

    public abstract String parseOut(Node tree);
    
}
