
package softparsmap;


//import ooc.*;
import java.util.*;

/**
 * This program is used to compare the topology of gene trees from
 * two family groups.
 * If used without any arguments, the usage will be printed. Usage:
 * <pre>
 * Compute [property file] compare_gene_trees [first family group did] [second family group did]
 * </pre>
 */
public class ComputeCompareGeneTrees extends ComputeFamilyGroupPair {

    private SortedSet _families1 = new TreeSet();

    private SortedSet _families2 = new TreeSet();

    private SortedSet _oneConsistentTwo = new TreeSet();

    private SortedSet _twoConsistentOne = new TreeSet();

    public void beforeCompute(FamilyGroup familyGroup) {
	_families2.addAll(getFamilyGroup2().getFamilies());
    }
    
    public void computeFamily(Family first, FamilyGroup familyGroup, 
			      boolean verbose) throws Throwable {

	// if true, the first family does not exist in the second family group
	if (!_families2.contains(first)) {
	    _families1.add(first);
	    return;
	}
	
	// getting family and removing it from second family set
	Family second = (Family)_families2.tailSet(first).first();
	_families2.remove(second);

	// creating trees
	GeneNode g1 = first.getGeneTree();
	GeneNode g2 = second.getGeneTree();

	// is one consistent with two
	if (g1.isConsistentWith(g2)) {
	    _oneConsistentTwo.add(first);
	}

	// is one consistent with two
	if (g2.isConsistentWith(g1)) {
	    _twoConsistentOne.add(first);
	}

    }

    public void afterCompute(FamilyGroup familyGroup) {
	System.out.println("\n");

	System.out.println
	    ("Let A denote the gene tree from the first family group and B \n"+
	     "the gene tree from the second family group.");
	System.out.println
	    ("More information on splits and consistency can be found \n"+
	     "in the API documentation for class Node and Split.\n");
	
	System.out.println
	    ("For following pair of families the tree A is consistent with B:"+
	     "\n"+_oneConsistentTwo+"\n");

	System.out.println
	    ("For following pair of families the tree B is consistent with A:"+
	     "\n"+_twoConsistentOne+"\n");
	
	if (_families1.size()>0) {
	    System.out.println
		("\nWarning, following families exist in the first family "+
		 "group \nbut not in the second:\n  "+_families1);
	}
	if (_families2.size()>0) {
	    System.out.println
		("Warning, following families exist in the second family "+
		 "group \nbut not in the first:\n  "+_families2+"\n");
	}
	
    }  


}
