
package softparsmap;


import ooc.*;
import java.util.*;

/**
 * This program is used when computing a task on a pair of family groups. 
 * If used without any arguments, the usage will be printed. Usage:
 * <pre>
 * Compute [property file] [task did] [first family group did] [second family group did]
 * </pre>
 */
public abstract class ComputeFamilyGroupPair extends Compute {

    private FamilyGroup _familyGroup1 = null;
    private FamilyGroup _familyGroup2 = null;

    public FamilyGroup getFamilyGroup1() {
	return _familyGroup1;
    }

    public FamilyGroup getFamilyGroup2() {
	return _familyGroup2;
    }
    
    public void computeTask() {
	
	// Checking arguments
	if(!(numberArgs()==2 || numberArgs()==3)) {
	    System.out.println
		("Usage: Compute [property file] [task did] "
		 +"[first family group did] [second family group did] "+
		 "| verbose");
	    return;
	}
	String groupDid = getArg(0);
	
	// verbose ?
	boolean verbose = false;
	if (numberArgs()==3 && getArg(2).equals("verbose")) {
	    verbose = true;
	}
	
	// creating family group
	System.out.println("Initiating first family group ...");
	_familyGroup1 = (FamilyGroup)getContainer()
	    .getTag("family_group::"+getArg(0)).getTagInstance();
	Set families1 = _familyGroup1.getFamilies();

		
	// creating family group
	System.out.println("Initiating second family group ...");
	_familyGroup2 = (FamilyGroup)getContainer()
	    .getTag("family_group::"+getArg(1)).getTagInstance();
	//_familyGroup2 = getPropertyFile().getFamilyGroup(getArg(1));
	Set families2 = _familyGroup2.getFamilies();

	// if true, initiating data source
	if(initiateDataSource()) {
	    System.out.println("Initiating data sources ...");
	    _familyGroup1.getDataSource().initiate();
	    _familyGroup2.getDataSource().initiate();
	}

	// setup is finished
	System.out.println("Computing task ");

	// if true first family group is run
	if (getAttributeBoolean("compute_first_family_group")) {
	    step(families1, _familyGroup1, verbose);
	}

	// if true second family group is run
	if (getAttributeBoolean("compute_second_family_group")){ 
	    step(families2, _familyGroup2, verbose);
	}
    }
    private void step(Set families, FamilyGroup familyGroup, boolean verbose) {
	int pro = 0;
	
	// run before task
	beforeCompute(familyGroup);

	// running family
	for(Iterator i = families.iterator();i.hasNext();) {
	    Family family = (Family)i.next();
	    try {
		computeFamily(family, familyGroup, verbose);
	    } catch (Throwable e) {
		System.out.println("\nFor family "+family.getFamilyNumber()+
				   " following exception was thrown.");
		e.printStackTrace(System.out);
		System.out.println();
	    }
	    if (useProgressIndicator()) {
		pro = progress(pro, 100);
	    }
	}
	
	// run after task
	afterCompute(familyGroup);
    }
    
    /**
     * This method is called before any family from the family group
     * is computed.
     *
     * @param familyGroup is the family group to be run.
     */
    public void beforeCompute(FamilyGroup familyGroup) {
    }
 
    /**
     * Compute a family.
     *
     * @param family is the family that will be computed
     * @param familyGroup is the family group from which family comes.
     * @throws Throwable because exceptions are logged for every family
     *         but the task will continue on the rest of the family group.
     */
    public abstract void computeFamily(Family family, FamilyGroup familyGroup, 
				       boolean verbose) throws Throwable;
 
    /**
     * This method is called after all families from the family group
     * have been computed.
     *
     * @param familyGroup is the family group which was run.
     */
    public void afterCompute(FamilyGroup familyGroup) {
    }
    
}
