
package softparsmap;


import ooc.*;
import java.util.*;
import java.io.*;

/**
 * This task will map species from the species tree to the
 * the nodes of the gene tree for all families in a given family group.
 * The result will be written to one file per family. 
 */
public class ComputeMap extends ComputeFamilyGroup {

    private TreeParser getTreeParser() {
	return (TreeParser)getAttribute("tree_parser_out");
    }
    
    //private TreeParser
    
    public void computeFamily(Family family, boolean verbose)
	throws Throwable {

	// creating gene tree
	GeneNode geneTree = family.getGeneTree();

	// check so that gene tree is binary
	if(!geneTree.isBinaryTree()) {
	    throw new FatalException("Gene input tree is not binary.");
	}
	
	// creating species tree
	SpeciesNode tolTree = family.getFamilyGroup().getDataSource()
	    .getSpeciesTree(geneTree);
	
	// printing input trees
	if(verbose) {
	    System.out.println();
	    System.out.println("Family "+family.getFamilyNumber()+":\n");
	    System.out.println("Gene tree:\n"+
			       getTreeParser().parseOut(geneTree)+"\n"+
			       geneTree.toStringAll());
	    System.out.println("Species tree:\n"+
			       tolTree.toNewick()+"\n"+
			       tolTree.toStringAll());
	}

	// gene tree has to be binary
	if(!geneTree.isBinaryTree()) {
	    throw new FatalException("The gene tree has to be binary.");
	}
	
	// inferring duplication and gene loss
	Mutation mutation = tolTree.inferMutation
	    (geneTree, false, getInparalogous());
	if (getPrintFamilyInfo()) {
	    // print duplication
	    System.out.println("- "+mutation.toString());
	}
	// output tree is ready
	GeneNode outputTree = geneTree;

	// printing result trees
	if(verbose) {	    
	    System.out.println("Output Tree: \n"+
			       getTreeParser().parseOut(outputTree)+"\n"+
			       outputTree.toStringAll());
	}

	// writing to file
	FileWriter fw = new FileWriter
	    (getTargetFile("template_target", family.getFamilyNumber()) );
	for (Iterator i = getChildIteratorByName("tree_parser");i.hasNext();) {
	    TreeParser tp = (TreeParser)((Tag)i.next()).getTagInstance();
	    fw.write(tp.parseOut(outputTree)+"\n");
	}
	fw.close();
    }

}
