
package softparsmap;


/**
 *  This interface is responsible of providing the package
 *  with data regarding species trees and gene sequences.
 */
public interface DataSource {

    /**
     * Returns the property file.
     *
     * @return the property file
     * @see PropertyFile
     */
    public PropertyFile getPropertyFile();

    /**
     * Called by the program {@link ComputeFamilyGroup} or
     * {@link ComputeFamilyGroupPair} once if the method
     * {@link Compute#initiateDataSource} returns true, else it is
     * not called at all.
     */
    public void initiate();
    
    /**
     * Returns the root to a species tree such that all genes found
     * in the gene tree exist in this species tree.
     *
     * @param geneTree the root of the gene tree
     * @return the root to the species tree
     * @see SpeciesNode
     * @see GeneNode
     */
    public SpeciesNode getSpeciesTree(GeneNode geneTree);

    /**
     * Returns the name of the species for a given species node.
     *
     * @param speciesNode is the species node. 
     * @return the name of the species that is associated with
     *         the species node
     * @see SpeciesNode
     */
    public String getSpeciesName(SpeciesNode speciesNode);

    /**
     * Returns an instance that implements the interface
     * {@link SequenceData} for a given gene leaf.
     *
     * @param leaf the gene leaf.
     * @return an instance of {@link SequenceData}
     * @see SequenceData
     */
    public SequenceData getSequenceData(GeneLeaf leaf);
    
}
