
package softparsmap;


import java.util.*;

/**
 * This interface represents a set of gene families and it is
 * responsible of providing these families for a certain task.
 */
public interface FamilyGroup {

    /**
     * Returns the property file. Used in the family class to
     * access the property file.
     *
     * @return the property file
     * @see PropertyFile
     */
    public PropertyFile getPropertyFile();

    /**
     * Returns the name of the family group.
     *
     * @return the name of this family group
     */
    public String getGroupName();

    /**
     * Returns the data source that this family group is using.
     *
     * @return the data source that this family is using
     */
    public DataSource getDataSource();

    /**
     * Returns the parser that is used by this family group when
     * parsing in gene trees.
     *
     * @return a tree parser
     * @see TreeParser
     */
    //public TreeParser getTreeParserIn();

    /**
     * Returns an instance of {@link Inparalogous} that this family
     * group will use regarding inparalogous.
     *
     * @return a inparalogous instance
     * @see Inparalogous
     */
    //public Inparalogous getInparalogous();

    /**
     * Returns the set of {@link Family} instances that this this
     * family group is responsible for.
     *
     * @return a set of {@link Family} instances
     * @see Family
     */
    public Set getFamilies();

    /**
     * Returns the gene tree given a family number.
     *
     * @return the gene node which is the root of the gene tree
     * @see GeneNode
     */
    public GeneNode getGeneTree(Integer familyNumber);

}
