
package softparsmap;


import java.io.File;
import java.util.*;

/**
 * This program is used to infer duplication and loss for small
 * gene and species trees typed at the prompt. 
 * If used without any arguments, the usage will be printed.
 */
public class InferMutation {
    
    public static void main(String[] args) throws Exception {
	if(args.length!=2) {
	    System.out.println
		("Usage: InferMutation [species tree] [gene tree] \n"+
		 "  Try: InferMutation '(((1),(2,3)),(4))' "+
		 "'(((1,2)1.0,3)1.0,4)'");
	    return;
	}
	// assign trees
	String speciesTree = args[0];
	String geneTree = args[1];
	
	// get gene parser and datasource
	PropertyFile pf = new PropertyFile("softparsmap/def.xml");
 	DataSource ds = (DataSource)
	    pf.getTag("data_source::dummy").getTagInstance();
	
	// parsing trees
	GeneNode gr = null;
	TreeParser parser = null;
	try {
	    parser = (TreeParser)
		pf.getTag("tree_parser::newick").getTagInstance();
	    gr = (GeneNode)parser.parseIn(geneTree);
	} catch(Exception e1) { 
	    try {
		gr = (GeneNode)((TreeParser)pf.getTag
				("tree_parser::newick_edge_value"))
		    .parseIn(geneTree);
	    } catch(Exception e2) { 
		try {
		    gr = (GeneNode)((TreeParser)pf.getTag
				    ("tree_parser::newick_edge_value_leaves"+
				     "_too")).parseIn(geneTree);
		} catch(Exception e3) {					
		    throw new FatalException("Tried all existing parsers but "+
					     "faild to parse the gene tree.");
		}
	    }
	}
	Node.setNextUnique(Node.nextUnique()+10);
	SpeciesNode sr = SpeciesNode.parseNewick(ds, speciesTree);

	// Infering mutation
	Mutation mut = sr.inferMutation(gr, true, null);

	// print gene and species tree
	System.out.println("Species tree\n"+sr.toStringAll());
	System.out.println("Gene tree\n"+gr.toStringAll());

	// make sure that SL and m(g) has been calculated for all nodes
	calculateTree(gr);
	
	// printing result
	System.out.println("Total number of\n  duplication = "+
			   mut.getDuplication()+"\n  loss = "+
			   mut.getLoss()+"\n");	
    }

    private static void calculateTree(GeneNode n) {
	n.getSL();
	if(n.hasChildren()) {
	    for(Iterator i = n.getChildren().iterator();i.hasNext();) {
		calculateTree((GeneNode)i.next());
	    }
	}
    }
  

}
