
package softparsmap;

/**
 * This class contains the result from inferring gene duplications and loss.
 */
public class Mutation {

    private int dup = 0;

    private int loss = 0;

    /**
     * Creating a mutation given duplications and loss.
     *
     * @param dup is the number of duplication
     * @param loss is the number of loss
     */
    public Mutation(int dup, int loss) {
	this.dup = dup;
	this.loss = loss;
    }

    /**
     * Returns number of duplications.
     *
     * @return number of duplications
     */
    public int getDuplication() {
	return this.dup;
    }

    /**
     * Returns number of loss.
     *
     * @return the number of loss
     */
    public int getLoss() {
	return this.loss;
    }

    /**
     * Returns true if the given mutation is the same as this.
     *
     * @param mut is a mutation
     * @return true if the given mutation is the same as this.
     */
    public boolean isSame(Mutation mut) {
	return this.dup==mut.dup && this.loss==mut.loss;
    }

    /**
     * Returns true if this is worse then the given mutation.
     *
     * @param mutate is the given mutation
     * @return true if this is worse then the given mutation
     */
    public boolean isWorse(Mutation mutate) {
	    return this.dup>mutate.dup || this.loss>mutate.loss;
    }

    /**
     * Adds duplications to this mutation.
     *
     * @param dup is the number of duplications that will be added.
     */
    public void addDuplication(int dup) {
	this.dup += dup;
    }

    /**
     * Adds loss to this mutation.
     * 
     * @param loss is the number of loss that is added.
     */
    public void addLoss(int loss) {
	this.loss += loss;
    }

    /**
     * Returns a string that contains the mutation.
     *
     * @return a string that contains the mutation
     */
    public String toString() {
	return "duplication="+dup+", loss="+loss;
    }
}
