
package softparsmap;


import ooc.*;
import java.util.*;

public class PreferredTreeStandard extends PreferredTree {

    private GeneNode geneTree = null;
      
    public int compare(Object o1, Object o2) {
	GeneNode n1 = (GeneNode)o1;
	GeneNode n2 = (GeneNode)o2;
	//  highest resolution
	int r1 = n1.getNodes().size();
	int r2 = n2.getNodes().size();
	if(r1!=r2) {
	    return r2-r1;
	}
	//  most strong edges
	int re1 = n1.countStrongEdges();
	int re2 = n2.countStrongEdges();
	if(re1!=re2) {
	    return re2-re1;
	}
	// messure root distance, smallest wins
	int rd1 = getRootDistance(n1);
	int rd2 = getRootDistance(n2);
	if(rd1!=rd2) {
	    return rd1-rd2;
	}
	// they are equal
	return 0;
    }
    
    public String getDesc(GeneNode gt) {
	return "#nodes="+gt.getNodes().size()+
	    ", #strong_edges="+gt.countStrongEdges()+
	    ", root_distance="+getRootDistance(gt);
    }

    private int getRootDistance(GeneNode candidateTree) {
	if(!candidateTree.hasChildren()) {
	    return 0;
	}
	GeneNode cChild = (GeneNode)candidateTree.getChildren()
	    .iterator().next();
	//
	Set cChildLeaves = cChild.getLeaves();
	Set cChildLeaves2 = new HashSet(candidateTree.getLeaves());
	cChildLeaves2.removeAll(cChildLeaves);
	GeneNode in = getFreeGeneTree().findNodeForUnrootedSplit
	    (cChildLeaves, cChildLeaves2);
	/*
	  if(in==null) {
	  // should be done better !!!
	  return 0;
	  }
	*/
	int i = 0;
	while(!in.isRoot()) {
	    i++;
	    in = (GeneNode)in.getParent();
	}
	return i;
    }
    
}
