
package softparsmap;


/**
 * This interface is responsible of providing the package with
 * all relevant sequence data.
 */
public interface SequenceData {

    /**
     * Returns the identity number for this sequence.
     *
     * @return the identity number for this sequence
     */
    public int getIdNumber();
    
    /**
     * Returns true if the sequence is complete, otherwise false.
     *
     * @return true if sequence is complete
     */
    public boolean isCompleteSequence();

    /**
     * Returns the length of the sequence.
     *
     * @return the length of the sequence.
     */
    public int getSequenceLength();


    /**
     * Returns the species name that harbor this sequence.
     *
     * @return the species name that harbor this sequence
     */
    public String getSpeciesName();

    /**
     * Returns the GI number. For more information see
     * <a href="http://www.ncbi.nlm.nih.gov/Sitemap/samplerecord.html#GInB">
     *   NCBI
     * </a>. If the NCBI Taxonomy database is not used, let this method
     * return -1.
     *
     * @return the GI number
     */
    public int getGiNumber();
    
}
