
package softparsmap;


import java.io.*;

/**
 * This class is used by DataSourceXmlNcbiTaxonomy to extract sequence data
 * from one file in XML-format. The format is flexible within limits,
 * see the <a href="{@docRoot}/softparsmap/def.xml">def.xml</a>
 * file for more information.
 */
public class SequenceDataXml extends AbstractSequenceData {

    private long _nextPointer = -1;
    
    private RandomAccessFile _dbFile = null;

    private String _data = null;

    public void setDatabaseFile(RandomAccessFile raf) {
	_dbFile = raf;
	_nextPointer = 0;
    }

    public boolean moveNextSequence() {
	//System.out.println("*** Enter moveNextSequence()");
	// erase old data
	_data = null;
	//_data_search = null;
	
	// make sure that file has been set
	if(_dbFile==null) {
	    throw new FatalException("The database file has not been set.");
	}

	try {
	    // set pointer to right poss
	    _dbFile.seek(_nextPointer);
	    
	    // try to find
	    byte[] buf = new byte
		[getAttributeInteger("file_reading_buffert_size")];
	    StringBuffer ori = new StringBuffer();
	    //StringBuffer searchIn = new StringBuffer();
	    String begin = "<"+getAttributeString("main_tag_name")+">";
	    String end = "</"+getAttributeString("main_tag_name")+">";
	    int pBegin = -1;
	    int pEnd = -1;	    
	    int numberBytes = 10;
	    int totalNumberBytes = 0;
	    boolean conti = true;
	    while(!(pBegin>=0 && pEnd>=0) && numberBytes>0) {
		numberBytes = _dbFile.read(buf);
		for(int i=0;i<numberBytes;i++) {
		    //char c = (char)buf[i];
		    ori.append((char)buf[i]);
		    //searchIn.append(String.valueOf(c).toLowerCase());
		}
		totalNumberBytes += numberBytes;
		if(pBegin<0) {
		    pBegin = ori.indexOf(begin);
		}
		if(pEnd<0 && pBegin>=0) {
		    pEnd = ori.indexOf(end, pBegin);
		}
		//System.out.println("* pBegin="+pBegin+", pEnd="+pEnd+
		//  ", numberBytes="+numberBytes);
	    }

	    // can be removed checking
	    if( numberBytes>0 && totalNumberBytes!=ori.length() ) {
		throw new FatalException("Wrong lengths in readin.");
	    }

	    // checking begin and end tag
	    if(pBegin<0 || pEnd<0 || pBegin>=pEnd) {
		if(numberBytes>0) {
		    throw new FatalException
			("There is something wrong with the database.");
		}
		// there are no move sequences
		return false;
		
	    }
	    
	    // setting data
	    pEnd = pEnd + end.length();
	    _data = ori.substring(pBegin, pEnd);
	    //_data_search = searchIn.substring(pBegin, pEnd);

	    //System.out.println("* _data='"+_data+"'");
	    
	    // setting next pointer
	    _nextPointer += pEnd;
	    
	    // just moved to next
	    return true;
	    
	} catch(IOException e) {
	    throw new FatalException(e);
	}
    }

    public FilePointerPart newFilePointerPartCurrent() {
	return new FilePointerPart
	    (_nextPointer-_data.length(), _data.length());
    }
    
    public void parseSequence(RandomAccessFile raf, FilePointerPart fpp) {
	try {
	    raf.seek(fpp.getPointer());
	    byte[] buf = new byte[fpp.getLength()];
	    int numberBytes = raf.read(buf);
	    StringBuffer ori = new StringBuffer();
	    //StringBuffer search = new StringBuffer();
	    for(int i=0;i<numberBytes;i++) {
		//char c = (char)buf[i];
		ori.append((char)buf[i]);
		//search.append(String.valueOf(c).toLowerCase());
	    }
	    _data = ori.toString();
	    //_data_search = search.toString();
	} catch(IOException e) {
	    throw new FatalException(e);
	}
    }

    public int getIdNumber() {
	return Integer.parseInt
	    (extractData(getAttributeString("sequence_id_tag_name"),
			 getAttributeString("sequence_id_template"),
			 getAttributeString("sequence_id_marker")));
    }
        
    public boolean isCompleteSequence() {
	return getAttributeString("complete_name").equals
	    (extractData(getAttributeString("partial_complete_tag_name"),
			 getAttributeString("partial_complete_template"),
			 getAttributeString("partial_complete_marker")));
    }
    
    public int getGiNumber() {
	return Integer.parseInt
	    (extractData(getAttributeString("gi_number_tag_name"),
			 getAttributeString("gi_number_template"),
			 getAttributeString("gi_number_marker")));
    }
    
    public int getSequenceLength() {
	return getTag(getAttributeString("sequence_tag_name")).length();
    }
    
    public String getSpeciesName() {
	return extractData(getAttributeString("organism_name_tag_name"),
			   getAttributeString("organism_name_template"),
			   getAttributeString("organism_name_marker"));
    }

    protected String extractData(String tag, String template, String marker) {
	StringTemplate st = new StringTemplate
	    (template, getAttributeString("markers_begin"),
	     getAttributeString("markers_end"));
	st.paintTemplate(getTag(tag));
	return st.getFirstValue(marker);
    }

    private String getTag(String tag) {
        int p0 = _data.indexOf("<" + tag + ">");
        int p1 = _data.indexOf("</" + tag + ">");
        if(p0<p1) {
            return _data.substring(p0+tag.length()+2,p1).trim();
        } else {
            throw new FatalException
                ("Could not find tag '"+tag+"' in sequence tag '"+_data+"'.");
        }
    }
    
}
