
package softparsmap;


import java.util.*;

/**
 * This is a sorted collection.
 */
public class SortedCollection implements Collection {

    private List coll;

    private Comparator comp;

    /**
     * Constructs an instance of this collection. 
     */
    public SortedCollection() {
	this.coll = new Vector();
    }

    /**
     * Constructs an instance of this collection using the given
     * comparator to keep the elements sorted.
     *
     * @param usedWhenSorting is a comparator used to keep the elements
     *        sorted.
     */
    public SortedCollection(Comparator usedWhenSorting) {
	this();
	this.comp = usedWhenSorting;
    }

    /**
     * Returns the comparator used when keeping the elements sorted.
     *
     * @return the comparator used when sorting
     */
    public Comparator getComparator() {
	if(this.comp==null) {
	    this.comp = new Comparator() {
		    public int compare(Object o1, Object o2) {
			return ((Comparable)o1).compareTo(o2);
		    }
		    public boolean equals(Object obj) {
			return false;
		    }
		};
	}
	return this.comp;
    }
    
    public boolean add(Object o) {
	for(int i=0;i<size();i++) {
	    if(getComparator().compare(o,this.coll.get(i))<=0) {
		this.coll.add(i,o);
		return true;
	    }
	}
	this.coll.add(o);
	return true;
    }
    
    public boolean addAll(Collection c) {
	boolean changed = false;
	for(Iterator i=c.iterator();i.hasNext();) {
	    if(add(i.next())) {
		changed = true;
	    }
	}
	return true;
    }

    public Object first() {
	return this.coll.get(0);
    }

    public Object last() {
	return this.coll.get(this.coll.size()-1);
    }
    
    public void clear() {
	this.coll.clear();
    }
    
    public boolean contains(Object o)  {
	return this.coll.contains(o);
    }
    
    public boolean containsAll(Collection c) {
	return this.coll.containsAll(c);
    }
    
    public boolean equals(Object o) {
	if(o instanceof SortedCollection) {
	    return this.coll.equals( ((SortedCollection)o).coll );
	}
	return false;
    }
    
    public int hashCode() {
	return this.coll.hashCode();
    }
    
    public boolean isEmpty() {
	return this.coll.isEmpty();
    }
    
    public Iterator iterator() {
	return this.coll.iterator();
    }
    
    public boolean remove(Object o) {
	return this.coll.remove(o);
    }
    
    public boolean removeAll(Collection c) {
	return this.coll.removeAll(c);
    }
    
    public boolean retainAll(Collection c) {
	return this.coll.retainAll(c);
    }
    
    public int size() {
	return this.coll.size();
    }
    
    public Object[] toArray() {
	return this.coll.toArray();
    }
    
    public Object[] toArray(Object[] a) {
	return this.coll.toArray(a);
    }

    public String toString() {
	return this.coll.toString();
    }

}
