
package softparsmap;


import ooc.*;

/**
 * This is a edge type that can me configured within limits.
 * See the <a href="{@docRoot}/softparsmap/def.xml">def.xml</a>
 * file for more information. 
 */
public class StandardEdgeType extends AbstractEdgeType {

    private int divide = 0;

    private double limit = Double.MIN_VALUE;
    
    private boolean divideValue() {
	if(this.divide==0) {
	    if(getAttributeBoolean("divide_value")) {
		this.divide = 1;
	    } else {
		this.divide = -1;
	    }
	}
	return this.divide>0;
    }

    private double getLimit() {
	if(this.limit==Double.MIN_VALUE) {
	    this.limit = getAttributeDouble("value_limit");
	}
	return this.limit;
    }

    /**
     * Sets the limit that defines if an edge value is weak or not.
     *
     * @param newLimit is the new limit
     */
    public void setLimit(double newLimit) {
	this.limit = newLimit;
    }
    
    public boolean isEdgeWeak(GeneNode node) {
	return node.getEdgeValue()<getLimit();
    }
    
    public double splitEdge(GeneNode n) {
	if(divideValue()) {
	    return n.getEdgeValue()/2.0;
	} else {
	    return n.getEdgeValue();
	}
    }

    public double joinEdges(GeneNode n1, GeneNode n2) {
	if(!n1.getEdgeType().equals(n2.getEdgeType())) {
	    throw new FatalException
		("The type of the edges has to be the same.");
	}
	if(divideValue()) {
	    return n1.getEdgeValue() + n2.getEdgeValue();
	} else {
	    if(n1.getEdgeValue()<n2.getEdgeValue()) {
		return n1.getEdgeValue();
	    } else {
		return n2.getEdgeValue();
	    }
	}
    }
    
    public String getName() {
	if(hasAttributeValue("name")) {
	    return getAttributeString("name");
	}
	return getAttributeString("did");
    }

    public String getShortName() {
	return getAttributeString("short_name");
    }

    /**
     * Returns the name, limit, and divide value.
     *
     * @return the name, limit, and divide value
     */
    public String toStringAll() {
	return getName()+", limit="+getLimit()+", divideValue="+divideValue();
    }
    
}
