package softparsmap;

import ooc.*;
import java.util.List;
import java.util.Map;

public class StringGeneNodeConverterSchreiberLabel 
    extends StringGeneNodeConverterStringTemplate { 

    private EdgeType _edgeType = null;
    private String _left = null;
    private String _right = null;

    public void setupInstance() {
	super.setupInstance();
	_edgeType = (EdgeType)getAttribute("edge_type");
	_left = getAttributeString("left_linked_node");
	_right = getAttributeString("right_linked_node");
    }

    public void beforeParsing(TreeStructureParser tsp, 
			      String treeString, Node tree) {
	super.beforeParsing(tsp, treeString, tree);
	if (!(tsp instanceof TreeStructureParserSchreiber)) {
	    throw new FatalException("Tag "+getTagRef()+": This node "+
				     "converter can only be used by the "+
				     "schreiber tree structure parser.");
	}
	if (tree!=null && !(tree instanceof GeneNode)) {
	    throw new FatalException("Tag "+getTagRef()+": This node "+
				     "converter can only handle gene trees.");
	}
    }

    public Node convertCoreToNode(String string, Object info) {
	if (info==null || !(info instanceof List)) {
	    throw new FatalException
		("Tag "+getTagRef()+": This node converter can only be used "+
		 "by the Schreiber parser.");
	}
	// if true, getting an existing node
	if (string!=null) {
	    int p0 = string.indexOf(_left);
	    int p1 = string.indexOf(_right);
	    int nodeId = Integer.parseInt
		(string.substring(p0+_left.length(), p1).trim());
	    return (Node)((List)info).get(nodeId-1);
	}
	// creating a new internal node
	return GeneNode.nextNode(_edgeType);
    }
    
    public Node convertCoreToLeaf(String string, Object info) {
	if (string==null) {
	    return GeneLeaf.nextLeaf(_edgeType);
	}
	return new GeneLeaf(_edgeType, Integer.parseInt(string));
    }
    
    public String convertCoreFromNode(Node node, Object info) {
	if (info==null || !(info instanceof Map)) {
	    throw new FatalException
		("Tag "+getTagRef()+": This node converter can only be used "+
		 "by the Schreiber parser.");
	}
	return "["+((Map)info).get(node).toString()+"]";
    }
    
    public String convertCoreFromLeaf(Node leaf, Object info) {	
	return new Integer(leaf.getLabel()).toString();
    }

}
