
package softparsmap;


import java.io.*;

/**
 * This interface is used to parse in and out trees. In order to define
 * your own parser you can extend the class {@link AbstractTreeParser}
 * and make the necessary changes in your property file.
 */
public interface TreeParser {

    /**
     * Returns the root of the parsed tree. The tree is parsed
     * from a input stream.
     *
     * @param tree is the input stream from which the tree is parsed
     * @return the root of the parsed tree
     * @see Node
     */
    public Node parseIn(InputStream tree);

    /**
     * Returns the root of the parsed tree. The tree is parsed
     * from a string.
     *
     * @param tree is the string from which the tree is parsed
     * @return the root of the parsed tree
     * @see Node
     */
    public Node parseIn(String tree);

    /**
     * Adding information to the existing tree. The parser extracts
     * the node information and adds it to the relevent node by 
     * mapping the key to the node information in the relevent node 
     * map.
     *
     * @param tree is the existing tree.
     * @param treeInfo is a stream in a tree format containing the
     *                 information to be added.
     */
    public void parseIn(Node tree, InputStream treeInfo);

    /**
     * Adding information to the existing tree. The parser extracts
     * the node information and adds it to the relevent node by 
     * mapping the key to the node information in the relevent node 
     * map.
     *
     * @param tree is the existing tree.
     * @param treeInfo is a string in a tree format containing the
     *                 information to be added.
     */
    public void parseIn(Node tree, String treeInfo);

    /**
     * This method will parse out the tree to the output stream.
     *
     * @param outputStream is the stream to which the tree will be parsed
     *        out to.
     * @param tree is the tree that are parsed out
     * @see Node
     */
    public void parseOut(OutputStream outputStream, Node tree);

    /**
     * This method will parse out the tree to a string. 
     *
     * @param tree is the tree that is parsed out
     * @see Node
     */
    public String parseOut(Node tree); 
    
}
