package softparsmap;


import ooc.*;
import java.util.*;

/**
 * This parser uses two complementary parts to parse in and out trees. 
 * The tree_structure_parser is parsing the tree structure and calls 
 * the node_converter when parsing in or out a node. See the 
 * <a href="{@docRoot}/softparsmap/def.xml">def.xml</a> file for more 
 * information.
 */
public class TreeParserDual extends AbstractTreeParser {

    private TreeStructureParser _structureParser = null;
    private StringNodeConverter _nodeConverter = null;
    private String _appendBeforeBody = null;
    private String _appendAfterBody = null;

    public void setupInstance() {
	super.setupInstance();
	_structureParser = 
	    (TreeStructureParser)getAttribute("tree_structure_parser");
	_nodeConverter = (StringNodeConverter)getAttribute("node_converter");
	_appendBeforeBody = getAttributeString("append_before_body");
	_appendAfterBody = getAttributeString("append_after_body");
    }

    public Node parseIn(String tree) {
	try {
	    _nodeConverter.beforeParsing(_structureParser, tree, null); 
	    Node root = 
		_structureParser.parseStructureIn(tree, _nodeConverter);
	    _nodeConverter.afterParsing(_structureParser, tree, root);
	    return root;
	} catch (Throwable e) {
	    throw new FatalException
		("Parser "+getTagRef()+": Failed to parser tree '"+tree+
		 "'.", e);
	}
    }
    
    public void parseIn(Node tree, String treeInfo) {
	try {
	    _nodeConverter.beforeParsing(_structureParser, treeInfo, tree); 
	    _structureParser.parseStructureIn(tree, treeInfo, _nodeConverter);
	    _nodeConverter.afterParsing(_structureParser, treeInfo, tree); 
	} catch (Throwable e) {
	    throw new FatalException("Parser "+getTagRef()+
				     ": Failed to parser tree info '"+
				     treeInfo+"'.", e);
	}
    }
    
    public String parseOut(Node tree) {
	try {
	    _nodeConverter.beforeParsing(_structureParser, null, tree); 
	    String ret = _appendBeforeBody +
		_structureParser.parseStructureOut(tree, _nodeConverter) +
		_appendAfterBody;
	    _nodeConverter.afterParsing(_structureParser, null, tree); 
	    return ret;
	} catch (Throwable e) {
	    throw new FatalException("Parser "+getTagRef()+
				     ": Failed to parser out tree.", e);
	}
    }
    
}
