
package softparsmap;

import ooc.*;
import java.util.*;

public class TreeStructureParserSchreiber extends LinkedTagInstance 
    implements TreeStructureParser {

    public static String ROOT_NODES_MAP_KEY = "schreiber_nodes";

    protected String _beforeCore = null;
    protected String _afterCore = null;
    protected String _left = null;
    protected String _right = null;
    protected String _nodeDivider = null;
    protected String _childNodeDivider = null;

    public void setupInstance() {
	super.setupInstance();
	_beforeCore = getAttributeString("before_core");
	_afterCore = getAttributeString("after_core");
	_left = getAttributeString("left_node_marker");
	_right = getAttributeString("right_node_marker");
	_nodeDivider = getAttributeString("node_divider");
	_childNodeDivider = getAttributeString("child_node_divider");
    }



    public Node parseStructureIn(String tree, StringNodeConverter converter) {
	// remove any text before and after the core
	String core = getCore(tree);
	// stepping trough the nodes
	List nodes = new Vector();
	Node parent = null;
	int p0 = core.indexOf(_left, 0);
	int p1 = nextEven(core, p0);
	while (-1<p0 && p0<p1) {
	    parent = converter.convertToNode(null, nodes);
	    nodes.add(parent);
	    StringTokenizer st = new StringTokenizer
		(core.substring(p0+_left.length(), p1), _childNodeDivider);
	    while (st.hasMoreTokens()) {
		String nodeString = st.nextToken();
		int pp0 = nodeString.indexOf(_left);
		Node child = null;
		if (pp0>-1) {
		    child = converter.convertToNode(nodeString, nodes);
		} else {
		    child = converter.convertToLeaf(nodeString, nodes);
		}
		parent.addChild(child);
	    }
	    p0 = core.indexOf(_left, p1+_right.length());
	    if (p0>-1) {
		p1 = nextEven(core, p0);
	    }
	}
	// if there is only two nodes then return just the leaf
	if (parent.numChildren()==1 && parent.getNodes().size()==2) {
	    Node leaf = (Node)parent.getChildren().iterator().next();
	    parent.removeChild(leaf);
	    return leaf;
	}
	// adding the node list to the root 
	parent.getMap().put(ROOT_NODES_MAP_KEY, nodes);
	//
	return parent;
    }
    // remove any text before and after the core
    protected String getCore(String tree) {
	int p0 = tree.indexOf(_beforeCore);
	int p1 = tree.lastIndexOf(_afterCore);
	if (!(p0>-1 && p0<p1)) {
	    throw new FatalException("Can not find the start and end of "+
				     "tree '"+tree+"' when parsing it.");
	}
	return tree.substring(p0+_beforeCore.length(), p1);
    }
    protected int nextEven(String core, int p0) {
	/* 
            [  [8  ], [9  ]  ]    count
         a  p0 p1  p2             2    
	 b     p0  p2 p1          1
	 c         p0 p1  p2      2
	 d            p0  p2      1
	 e                p0 p2   0  
         returns             p0
	*/
	int count = 1;
	int p1, p2;
	while (count>0) {
	    p1 = core.indexOf(_left, p0+_left.length());
	    p2 = core.indexOf(_right, p0+_left.length());
	    if (p2<0) {
		throw new FatalException("Tag "+getTagRef()+
					 ": Should not be possible.");
	    }
	    if (p0<p1 && p1<p2) {
		count++;
		p0 = p1;
	    } else {
		count--;
		p0 = p2;
	    }
	}
	return p0;
    }
    


    public void parseStructureIn(Node tree, String treeInfo,
				 StringNodeConverter converter) {
	// remove any text before and after the core
	String core = getCore(treeInfo);
	// if it is only a leaf
	if (converter.isLeaf(tree)) {
	    converter.convertToLeaf(getCore(core), tree);
	    return;
	}
	// stepping trough the nodes
	List nodes = (List)tree.getMap().get(ROOT_NODES_MAP_KEY);
	Node parent = null;
	int p0 = core.indexOf(_left, 0);
	int p1 = nextEven(core, p0);
	int i = 0;
	while (-1<p0 && p0<p1) {
	    //System.out.println("### core='"+core+"', p0="+p0+", p1="+p1);
	    parent = (Node)nodes.get(i);
	    StringTokenizer st = new StringTokenizer
		(core.substring(p0+_left.length(), p1), _childNodeDivider);
	    Iterator children = parent.getChildren().iterator();
	    while (st.hasMoreTokens()) {
		String nodeString = st.nextToken();
		int pp0 = nodeString.indexOf(_left);
		Node child = (Node)children.next();
		if (pp0>-1) {
		    converter.convertToNode(nodeString, child);
		} else {
		    converter.convertToLeaf(nodeString, child);
		}
	    }
	    p0 = core.indexOf(_left, p1+_right.length());
	    if (p0>-1) {
		p1 = nextEven(core, p0);
	    }
	    i++;
	}
    }
 


    public String parseStructureOut(Node tree, StringNodeConverter converter) {
	// if the tree is a leaf
	if (converter.isLeaf(tree)) {
	    return _left + _left + converter.convertFromLeaf(tree, null) + 
		_right + _right;
	}
	// else 
	StringBuffer res = new StringBuffer();
	res.append(_beforeCore);
	if (tree.hasChildren()) {
	    parseStructureOut(tree, res, new Hashtable(), 1, converter);
	}
	res.append(_afterCore);
	return res.toString();
    }
    protected int parseStructureOut(Node node, StringBuffer tree, 
				    Map nodeToNumber, int nextNumber, 
				    StringNodeConverter converter) {
	// if leaf, do nothing
	if (!node.hasChildren()) {
	    return nextNumber;
	}
	// deep first
	for (Iterator i = node.getChildren().iterator();i.hasNext();) {
	    nextNumber = parseStructureOut
		((Node)i.next(), tree, nodeToNumber, nextNumber, converter);
	}
	// building string for one internal node
	tree.append(_left);
	for (Iterator i = node.getChildren().iterator();i.hasNext();) {
	    Node child = (Node)i.next();
	    if (converter.isLeaf(child)) {
		// adding leaf data
		tree.append(converter.convertFromLeaf(child, nodeToNumber));
	    } else {
		// adding internal node
		tree.append(converter.convertFromNode(child, nodeToNumber));
	    }
	    // adding child divider
	    if (i.hasNext()) {
		tree.append(_childNodeDivider);
	    }
	}
	tree.append(_right);
	if (!node.isRoot()) {
	    tree.append(_nodeDivider);
	}
	// adding this node to the map
	nodeToNumber.put(node, new Integer(nextNumber));
	nextNumber++;
	//
 	return nextNumber;
    }

}
