package softparsmap.tests;


import junit.framework.*;
import softparsmap.*;
import ooc.*;

public abstract class CommonTestCase extends TestCase {

    public PropertyFile pf = null;

    public EdgeType et = null;

    public StandardEdgeType etML = null;
    public StandardEdgeType etBO = null;
    public StandardEdgeType etUN = null;
    //public StandardEdgeType etRO = null;

    public DataSource ds = null;
    
    protected void setUp() throws Exception {
	if(this.pf==null) {
	    this.pf = new PropertyFile("softparsmap/tests/tests.xml"); 
	}
	if(this.ds==null) {
	    this.ds = (DataSource)
		pf.getTag("data_source::dummy").getTagInstance();
	}
	
	if(this.et==null) {
	    this.et = (EdgeType)pf.getTag("edge_type::unknown")
		.getTagInstance();
	}
	etML = (StandardEdgeType)
	    pf.getTag("edge_type::maximum_likelihood").getTagInstance();
	etML.setLimit(0.1);
	etBO = (StandardEdgeType)
	    pf.getTag("edge_type::bootstrap").getTagInstance();
	etBO.setLimit(90.0);
	etUN = (StandardEdgeType)
	    pf.getTag("edge_type::unknown").getTagInstance();
	etUN.setLimit(0.5);
	
	//etRO = (StandardEdgeType)pf.getEdgeType("root");
	
    }

    public GeneNode parseNewick(EdgeType edgeType, String tree) {
	TreeParserNewick tp = (TreeParserNewick)
	    pf.getTag("tree_parser::bootstrap").getTagInstance();
	EdgeType oe = tp.getEdgeType();
	tp.setEdgeType(edgeType);
	GeneNode ret = (GeneNode)tp.parseIn(tree);
	tp.setEdgeType(oe);
	return ret;
    }

    public GeneNode parseNewickNe(EdgeType edgeType, String tree) {
	TreeParserNewick tp = (TreeParserNewick)
	    pf.getTag("tree_parser::newick").getTagInstance();
	EdgeType oe = tp.getEdgeType();
	tp.setEdgeType(edgeType);
	GeneNode ret = (GeneNode)tp.parseIn(tree);
	tp.setEdgeType(oe);
	return ret;
    }

    public GeneLeaf newLeaf(int label) {
	return new GeneLeaf(et, label);
    }

    public SpeciesLeaf nextLeafS(int geneLeaf) {
	return SpeciesLeaf.nextLeafS(ds).addSequence(et, geneLeaf);
    }
    
    public SpeciesNode nextNodeS() {
	return SpeciesNode.nextNodeS(ds);
    }
    
    public SpeciesNode triS(SpeciesNode n1, SpeciesNode n2, SpeciesNode n3)  {
	return (SpeciesNode)nextNodeS().addChild(n1).addChild(n2).addChild(n3);
    }

    public SpeciesNode triS(int n1, int n2, int n3) {
	return triS(nextLeafS(n1), nextLeafS(n2), nextLeafS(n3));
    }
    
    public SpeciesNode binS(SpeciesNode n1, SpeciesNode n2) {
	return (SpeciesNode)nextNodeS().addChild(n1).addChild(n2);
    }

    public SpeciesNode binS(int n1, SpeciesNode n2) {
    	return binS(nextLeafS(n1),n2);
    }
    
    public SpeciesNode binS(SpeciesNode n1, int n2) {
	return binS(n1,nextLeafS(n2));
    }
    
    public SpeciesNode binS(int n1, int n2)  {
	return binS(nextLeafS(n1), nextLeafS(n2));
    }


    
    public GeneNode tri(GeneNode n1, GeneNode n2, GeneNode n3)  {
	return (GeneNode)GeneNode.nextNode(et).addChild(n1).addChild(n2).addChild(n3);
    }
    
    public GeneNode tri(int n1, int n2, int n3)  {
	return tri(new GeneLeaf(et, n1), new GeneLeaf(et, n2), new GeneLeaf(et, n3));
    }

    public GeneNode bin(GeneNode n1, GeneNode n2)  {
	return (GeneNode)GeneNode.nextNode(n1.getEdgeType()).addChild(n1).addChild(n2);
    }
    
    public GeneNode bin(int n1, GeneNode n2)  {
    	return bin(new GeneLeaf(n2.getEdgeType(), n1),n2);
    }

    public GeneNode bin(GeneNode n1, int n2)  {
	return bin(n1,new GeneLeaf(n1.getEdgeType(), n2));
    }

    public GeneNode bin(int n1, int n2)  {
	return bin(new GeneLeaf(et, n1), new GeneLeaf(et, n2));
    }

}
