
package softparsmap.tests;



import java.util.*;

public class DescDup {

    public static void main(String[] args) {
	if(args.length<2) {
	    System.out.println("DescDup [number of times] [SL(g_1)] "+
			       "[SL(g_2)] ... [SL(g_n)]");
	    return;
	}

	int d = new DescDup(args).countDuplications(Integer.parseInt(args[0]));
	if(d<0) {
	    System.out.println("Failed to count number of duplications.");
	} else {
	    System.out.println("Number of duplications = "+d);
	}
	     
    }

    private Set _geneNodes = null;
    
    public DescDup(String[] args) {
	// creating the child set using the command line arguments.
	_geneNodes = new HashSet();
	for(int i=1;i<args.length;i++) {
	    StringTokenizer st = new StringTokenizer(args[i],",");
	    Set species = new HashSet();
	    while (st.hasMoreTokens()) {
		species.add(st.nextToken().trim());
	    }
	    _geneNodes.add(new GeneNode(species));
	}
    }
    

    /**
     *  This method will count the number of duplications that
     *  a gene node g will cause when g's child set is _geneNodes.
     */
    public int countDuplications(int tryRandomTimes) {
	
	// sort gene nodes such that g_j is before g_i in the
	// list if |SL(g_i)|<=|SL(g_j)|
	Comparator comp = new Comparator() {
		public int compare(Object o1, Object o2) {
		    return
			((GeneNode)o2).getSpecies().size() -
			((GeneNode)o1).getSpecies().size();
		}
	    };
	Object[] tempSort = _geneNodes.toArray();
	Arrays.sort(tempSort,comp);
    	List sortedGeneNodes = Arrays.asList(tempSort);
	
	// stepping through sortedGeneNodes and adding them to buckets
	List buckets = new Vector();
	for(int i=0;i<sortedGeneNodes.size();i++) {
	    GeneNode gi = (GeneNode)sortedGeneNodes.get(i);
	    addGiToFirstBucket(buckets, gi);
	}

	// first time the sorted method will be used, then the
	// random method
	int numberOfTimes = 0;
	do {
	    System.out.println("Built buckets "+buckets);
	    
	    // try to verify the minimum size by common species 
	    if(verifyByCommonSpecies(buckets)) {
		System.out.println("Verified with common species");
		return buckets.size()-1;
	    }
	    
	    // try to verify the minimum size by common species sets
	    if(verifyByCommonSpeciesSets(buckets)) {
		System.out.println("Verified with common species sets");
		return buckets.size()-1;
	    }
	    System.out.println("Failed to verify buckets");
	    
	    // creating a new buckets using the random method
	    buckets.clear();
	    List tempGeneNodes = new Vector(_geneNodes);
	    while(tempGeneNodes.size()>0) {
		int i = (int)(Math.random()*(double)tempGeneNodes.size());
		GeneNode gi = (GeneNode)tempGeneNodes.remove(i);
		addGiToFirstBucket(buckets, gi);
	    }

	    // see if we should try it again.
	    numberOfTimes++;
	} while(numberOfTimes<tryRandomTimes);

	// failed to count number of duplications
	return -100;
    }

    
    
    // placing gi in the first buckit where for all gj in the
    // bucket SL(gi) is disjoint with SL(gj)
    public void addGiToFirstBucket(List buckets, GeneNode gi) {
	for(int b=0;b<buckets.size() && gi!=null;b++) {
	    List bucket = (List)buckets.get(b);
	    boolean giIsDisjointWithAll = true;
	    for(int j=0;j<bucket.size() && giIsDisjointWithAll;j++) {
		GeneNode gj = (GeneNode)bucket.get(j);
		if(gi.anySpeciesInCommon(gj)) {
		    giIsDisjointWithAll = false;
		}
	    }
	    if(giIsDisjointWithAll) {
		// add gi to bucket
		bucket.add(gi);
		gi = null;
	    }
	}
	if(gi!=null) {
	    // gi could not be added to any existing bucket. Adding
	    // a new bucket and adding gi to it.
	    List bucket = new Vector();
	    bucket.add(gi);
	    buckets.add(bucket);
	}
    }
    

    
    // Try to verify that the number of buckets can not be smaller
    // by examine if there is a common species in all buckets.
    public boolean verifyByCommonSpecies(List buckets) {
	Set commonSpecies = null;
	for(int b=0;b<buckets.size();b++) {
	    List bucket = (List)buckets.get(b);
	    Set speciesInBucket = new HashSet();
	    for(int j=0;j<bucket.size();j++) {		
		GeneNode gj = (GeneNode)bucket.get(j);
		speciesInBucket.addAll(gj.getSpecies());
	    }
	    if(commonSpecies==null) {
		commonSpecies = speciesInBucket;
	    } else {
		// Retains only the species in commonSpecies that
		// are contained in current bucket.
		commonSpecies.retainAll(speciesInBucket);
		if(commonSpecies.size()==0) {
		    // if commonSpecies is empty, there are no common species.
		    return false;
		}
	    }
	}
	return commonSpecies.size()>0;
    }


    
    // Try to verify that the number of buckets can not be smaller
    // by finding one species set in every bucket such that any two
    // sets of those are disjoint.
    public boolean verifyByCommonSpeciesSets(List buckets) {
	return recVerifyByCommonSpeciesSets(buckets, -1, new Vector());
    }
    private boolean recVerifyByCommonSpeciesSets(List buckets,
						 int lastBucketIndex,
						 List geneNodes) {
	if(lastBucketIndex+1>=buckets.size()) {
	    // a list of gene nodes has been found since the end
	    // of the buckets has been reached.
	    return true;
	}
	List currentBucket = (List)buckets.get(lastBucketIndex+1);
	for(int i=0;i<currentBucket.size();i++) {
	    GeneNode gi = (GeneNode)currentBucket.get(i);
	    // checking if gi has common species with all gene
	    // nodes in geneNodes
	    boolean commonSpeciesWithAll = true;
	    for(int j=0;j<geneNodes.size() && commonSpeciesWithAll;j++) {
		GeneNode gj = (GeneNode)geneNodes.get(j);
		if(!gi.anySpeciesInCommon(gj)) {
		    commonSpeciesWithAll = false;
		}
	    }
	    // if true gene node will be added and sent on in the recursion
	    if(commonSpeciesWithAll) {
		geneNodes.add(gi);
		if(recVerifyByCommonSpeciesSets
		   (buckets, lastBucketIndex+1, geneNodes)) {
		    return true;
		}
		// did not reach all the way so gi is removed
		geneNodes.remove(geneNodes.size()-1);
	    }
	}
	return false;
    }



    // this class is internal and will make the code in this class
    // independent from the package.
    public class GeneNode {

	public Set _species = null;

	public GeneNode(Set species) {
	    _species = species;
	}

	public Set getSpecies() {
	    return _species;
	}
	
	public boolean anySpeciesInCommon(GeneNode gi) {
	    Set tempSet = new HashSet(_species);
	    tempSet.retainAll(gi.getSpecies());
	    return tempSet.size()>0;
	}

	public String toString() {
	    return _species.toString();
	}
	
    }
    
    
}
