/*
 *  TestNodeLeaf.java  ;  ver 1.0  ; 2002-10-08
 *
 *
 */

package softparsmap.tests;
import java.util.*;
import java.io.*;
import softparsmap.*;
import junit.framework.*;

public class TestNodeLeaf extends CommonTestCase {

    public static void main(String[] args) {
	junit.textui.TestRunner.run(suite());
    }

    public static Test suite() {
	return new TestSuite(TestNodeLeaf.class);
    }

    public void testResolveUncertainties() {
	GeneNode nb,nu;
	// added 2004-04-27, allow non binary input trees
	nu = parseNewickNe(et, "(1,(2,3,(4,5,6,7)))");
	nb = parseNewickNe(et, "(1,(2,(3,(4,5,(6,7)))))");
	//System.out.println("\nOriginal Tree: \n"+nb.toStringAll());
	//System.out.println("\nUncertain Tree: \n"+nu.toStringAll());
	nu.resolveUncertainties(nb);
	//System.out.println("\nResolved Tree: \n"+nu.toStringAll());
	Assert.assertTrue(nu.findLeaf(4).getParent().numChildren()==3);
	
	// added 2004-04-04
	nb = parseNewick(et, "(1:1.0,(6:1.0,(8:1.0,(3:1.0,(16:1.0,(9:1.0,(5:1.0,((7:1.0,(2:1.0,(18:1.0,(17:1.0,(4:1.0,10:1.0):0.7):0.56):1.0):1.0):0.9,(19:1.0,((12:1.0,14:1.0):0.62,(15:1.0,(11:1.0,13:1.0):0.98):0.48):0.85):0.69):0.68):0.83):0.43):0.55):1.0):0.37):1.0);");
	nu = parseNewick(et, "(16:1.0,((7:1.0,(2:1.0,(18:1.0,(10:1.0,4:1.0):0.56):1.0):1.0):0.9,9:1.0,(((12:1.0,14:1.0,(11:1.0,13:1.0):0.98):1.0,15:1.0):0.85,8:1.0):1.0,(3:1.0,5:1.0):1.0,19:1.0):1.0);");
	//System.out.println("\nBinary Tree: \n"+nb.toStringAll());
	//System.out.println("\nUncertain Tree: \n"+nu.toStringAll());
	nu.resolveUncertainties(nb);
	//System.out.println("\nResolved Tree: \n"+nu.toStringAll());
	Assert.assertTrue(nu.findLeaf(7).isBinaryTree());

	
	// added 2004-04-04
	nb = parseNewickNe(et, "((((1,2),3),4),5)");
	nu = parseNewickNe(et, "((2,3,4),5)");
	// System.out.println("\nBinary Tree: \n"+nb.toStringAll());
	//System.out.println("\nUncertain Tree: \n"+nu.toStringAll());
	nu.resolveUncertainties(nb);
	//System.out.println("\nResolved Tree: \n"+nu.toStringAll());
	Assert.assertTrue(nu.isBinaryTree());
	
	// added 2004-03-21
	nb = parseNewickNe(et, "(((1,2),(3,4)),5)");
	nu = parseNewickNe(et, "((1,3,(2,4)),5)");
	//System.out.println("\nBinary Tree: \n"+nb.toStringAll());
	//System.out.println("\nUncertain Tree: \n"+nu.toStringAll());
	nu.resolveUncertainties(nb);
	//System.out.println("\nResolved Tree: \n"+nu.toStringAll());
	Assert.assertTrue(nu.rootedEquals
			  (parseNewickNe(et, "((1,3,(2,4)),5)")));

	// added 2004-03-21
	nb = parseNewickNe(et, "(((1,2),(3,4)),5)");
	nu = parseNewickNe(et, "((1,2,3,4),5)");
	//System.out.println("\nBinary Tree: \n"+nb.toStringAll());
	//System.out.println("\nUncertain Tree: \n"+nu.toStringAll());
	nu.resolveUncertainties(nb);
	//System.out.println("\nResolved Tree: \n"+nu.toStringAll());
	Assert.assertTrue(nb.rootedEquals(nu));
	
    }

    /*
    public void testFindNodeWithLeaves() {
	GeneNode n,n1,n2;
	n = bin(bin(bin(bin(1,2),3),4),5);
	n1 = (GeneNode)n.findLeaf(1).getParent();
	Assert.assertTrue(n.findNodeWithLeaves(n1.getLeaves()).equals(n1));
	n1 = (GeneNode)n.findLeaf(2).getParent();
	Assert.assertTrue(n.findNodeWithLeaves(n1.getLeaves()).equals(n1));
	n1 = (GeneNode)n.findLeaf(3).getParent();
	Assert.assertTrue(n.findNodeWithLeaves(n1.getLeaves()).equals(n1));
	n1 = (GeneNode)n.findLeaf(4).getParent();
	Assert.assertTrue(n.findNodeWithLeaves(n1.getLeaves()).equals(n1));

	n = bin(bin(bin(1,2),bin(3,4)),5);
	n1 = (GeneNode)n.findLeaf(1).getParent();
	Assert.assertTrue(n.findNodeWithLeaves(n1.getLeaves()).equals(n1));
	n1 = (GeneNode)n.findLeaf(3).getParent();
	Assert.assertTrue(n.findNodeWithLeaves(n1.getLeaves()).equals(n1));
	n1 = (GeneNode)n.findLeaf(5).getParent();
	Assert.assertTrue(n.findNodeWithLeaves(n1.getLeaves()).equals(n1));
    }
    */
       
    public void testFindLCA() {
	GeneNode n,n1;
	n = bin(1,bin(2,bin(3,bin(4,5))));

	n1 = n.findLeaf(1);
	Assert.assertTrue(n.findLCA(n1.getLeaves()).equals(n1));
	n1 = n.findLeaf(2);
	Assert.assertTrue(n.findLCA(n1.getLeaves()).equals(n1));
	n1 = n.findLeaf(3);
	Assert.assertTrue(n.findLCA(n1.getLeaves()).equals(n1));
	n1 = n.findLeaf(4);
	Assert.assertTrue(n.findLCA(n1.getLeaves()).equals(n1));
	n1 = n.findLeaf(5);
	Assert.assertTrue(n.findLCA(n1.getLeaves()).equals(n1));

	n = bin(bin(1,2),bin(3,4));
	Set s = new HashSet();
	s.add(new GeneLeaf(et,2));
	s.add(new GeneLeaf(et,3));
	Assert.assertTrue(n.findLCA(s).equals(n));

    }
    
    public void testChildIterator() {
	GeneNode n = bin(1,bin(2,bin(3,4)));
	int c = 0;
	for(Iterator i=n.getChildIterator();i.hasNext();) {
	    i.next();
	    c++;
	}
	Assert.assertTrue(c==2);
	((GeneNode)n.findLeaf(2).getParent()).setCollapsedEdge(true);
	c = 0;
	for(Iterator i=n.getChildIterator();i.hasNext();) {
	    i.next(); c++;
	}
	Assert.assertTrue(c==3);
	((GeneNode)n.findLeaf(3).getParent()).setCollapsedEdge(true);
	c = 0;
	for(Iterator i=n.getChildIterator();i.hasNext();) {
	    i.next(); c++;
	}
	Assert.assertTrue(c==4);	
    }
    
    public void testWeakEdge()  {
	GeneNode n1 = GeneNode.nextNode(et);
	//n1.setParent(GeneNode.nextNode(et));
	GeneNode p = GeneNode.nextNode(et);
	p.addChild(n1);
	// UN
	n1.setEdgeType(etUN);
	n1.setEdgeValue(1);
	Assert.assertTrue(!n1.isEdgeWeak());
	n1.setEdgeValue(0.1);
	Assert.assertTrue(n1.isEdgeWeak());
	n1.setEdgeValue(0.5);
	Assert.assertTrue(!n1.isEdgeWeak());
	// BO
	n1.setEdgeType(etBO);
	n1.setEdgeValue(100);
	Assert.assertTrue(!n1.isEdgeWeak());
	n1.setEdgeValue(1);
	Assert.assertTrue(n1.isEdgeWeak());
	n1.setEdgeValue(90);
	Assert.assertTrue(!n1.isEdgeWeak());
	// ML
	n1.setEdgeType(etML);
	n1.setEdgeValue(1);
	Assert.assertTrue(!n1.isEdgeWeak());
	n1.setEdgeValue(0.01);
	Assert.assertTrue(n1.isEdgeWeak());
	n1.setEdgeValue(0.1);
	Assert.assertTrue(!n1.isEdgeWeak());
	n1.setEdgeValue(0.051);
	Assert.assertTrue(n1.isEdgeWeak());
	n1.setEdgeValue(0.049);
	Assert.assertTrue(n1.isEdgeWeak());	
    }

    public void testMergeSplitEdges()  {
	GeneNode n, p;
	// split
	p = GeneNode.nextNode(et);
	n = (GeneNode)new GeneNode(etML,-1, 1.1);
	p.addChild(n);    
	Assert.assertTrue(n.getEdgeType().splitEdge(n)==1.1);
	p = GeneNode.nextNode(et);
	n = (GeneNode)new GeneNode(etBO,-1, 1.1);
	p.addChild(n);
	Assert.assertTrue(n.getEdgeType().splitEdge(n)==0.55);
	// merge
    }

    /*
    public void testPurge()  {
	GeneNode n1,n2;
	Set s1,s2;
	n2 = bin(1,2);
	n1 = bin(n2,3);
	// this
	s1 = n1.getLeaves();
	Assert.assertTrue(s1==n1.getLeaves());
	n1.purgeThis();
	Assert.assertTrue(s1!=n1.getLeaves());
	// purgeDown
	s2 = n2.getLeaves();
	Assert.assertTrue(s2==n2.getLeaves());
	n1.purgeDown();
	Assert.assertTrue(s2!=n2.getLeaves());
	// purgeUp
	s1 = n1.getLeaves();
	Assert.assertTrue(s1==n1.getLeaves());
	n2.purgeUp();
	Assert.assertTrue(s1!=n1.getLeaves());
    }
    public void testResetFlags()  {
	GeneNode n1,n2,n3;
	n1 = bin(1,2);
	n2 = bin(n1,3);
	n3 = bin(n2,4);
	// resetFlags
	//Assert.assertTrue(n3.levelOnFirstSpeciationEvent()==0);
	Assert.assertTrue(!n3.duplicatedNode());
	Assert.assertTrue(!n3.collapsedEdge());
	n3.setDuplicationsNode(1);
	n3.setCollapsedEdge(true);
	//Assert.assertTrue(n3.levelOnFirstSpeciationEvent()==1);
	Assert.assertTrue(n3.duplicatedNode());
	Assert.assertTrue(n3.collapsedEdge());
	n3.resetFlags();
	Assert.assertTrue(!n3.duplicatedNode());
	Assert.assertTrue(!n3.collapsedEdge());
	n3.setDuplicationsNode(1);
	n2.setDuplicationsNode(1);
	//Assert.assertTrue(n3.levelOnFirstSpeciationEvent()==2);
	n3.setDuplicationsNode(1);
	n2.setDuplicationsNode(1);
	n1.setDuplicationsNode(1);	
	//	System.out.println("\n"+n3.toStringAll());
	//	System.out.println(n3.levelOnFirstSpeciationEvent());	
	//Assert.assertTrue(n3.levelOnFirstSpeciationEvent()>
	//		  Integer.MAX_VALUE/2-1);
	
	// purgeDown
	n1.setDuplicationsNode(1);
	n1.setCollapsedEdge(true);
	n3.resetFlagsDown();
	Assert.assertTrue(!n1.duplicatedNode());
	Assert.assertTrue(!n1.collapsedEdge());
	// purgeUp
	n3.setDuplicationsNode(1);
	n3.setCollapsedEdge(true);
	n1.resetFlagsUp();
	Assert.assertTrue(!n3.duplicatedNode());
	Assert.assertTrue(!n3.collapsedEdge());
    }
    */

    /*
    public void testStrongerEdge()  {
	GeneNode n1 = (GeneNode)GeneNode.nextNode(et).setParent(GeneNode.nextNode(et));
	GeneNode n2 = (GeneNode)GeneNode.nextNode(et).setParent(GeneNode.nextNode(et));
	etUN.setLimit(100.0);
	etBO.setLimit(100.0);
	etML.setLimit(100.0);
	// same BO
	n1.setEdgeType(etBO);
	n1.setEdgeValue(10);
	n2.setEdgeType(etBO);
	n2.setEdgeValue(20);
	Assert.assertTrue(n2.strongerSupport(n1) && !n1.strongerSupport(n2));
	Assert.assertTrue(n2.strongestSupport(n1)==n2);
	// same ML
	n1.setEdgeType(etML);
	n1.setEdgeValue(10);
	n2.setEdgeType(etML);
	n2.setEdgeValue(20);
	Assert.assertTrue(n2.strongerSupport(n1) && !n1.strongerSupport(n2));
	Assert.assertTrue(n2.strongestSupport(n1)==n2);
	// BO against ML
	n1.setEdgeType(etBO);
	n1.setEdgeValue(10);
	n2.setEdgeType(etML);
	n2.setEdgeValue(20);
	Assert.assertTrue(n2.strongerSupport(n1) && !n1.strongerSupport(n2));
	Assert.assertTrue(n2.strongestSupport(n1)==n2);
	n1.setEdgeValue(20);
	n2.setEdgeValue(10);
	Assert.assertTrue(n2.strongerSupport(n1) && !n1.strongerSupport(n2));
	Assert.assertTrue(n2.strongestSupport(n1)==n2);
	n1.setEdgeValue(110);
	n2.setEdgeValue(20);
	Assert.assertTrue(!n2.strongerSupport(n1) && n1.strongerSupport(n2));
	n1.setEdgeValue(110);
	n2.setEdgeValue(120);
	Assert.assertTrue(n2.strongerSupport(n1) && !n1.strongerSupport(n2));
	Assert.assertTrue(n2.strongestSupport(n1)==n2);
    }
    */

    public void testGetNodes()  {
	GeneNode n1,n2,n3;
	n1 = bin(1,2);
	n2 = bin(n1,3);
	n3 = bin(n2,4);
	Set s = new TreeSet();
	s.add(n1);
	s.add(n2);
	s.add(n3);
	s.add(new GeneLeaf(et,1));
	s.add(new GeneLeaf(et,2));
	s.add(new GeneLeaf(et,3));
	s.add(new GeneLeaf(et,4));
	Assert.assertTrue(s.equals(n3.getNodes()));
    }

    public void testGetInternalNodes()  {
	GeneNode n1,n2,n3;
	n1 = bin(1,2);
	n2 = bin(n1,3);
	n3 = bin(n2,4);
	Set s = new TreeSet();
	s.add(n1);
	s.add(n2);
	s.add(n3);
	Assert.assertTrue(s.equals(n3.getInternalNodes()));
    }

    public void testCollapseEdges()   {
	GeneNode n1,n2,n3;
	// 2002-12-16,
	n1 = bin(bin(1,bin(2,3)),4);
	((GeneNode)n1.findLeaf(2).getParent()).setCollapsedEdge(true);
	n1.collapseEdges();
	n2 = bin(tri(1,2,3),4);
	Assert.assertTrue(n1.rootedEquals(n2));
	// init test
	n1 = bin(bin(1,2),3);
	n2 = n1.cloneTree();
	n2.collapseEdges();
	Assert.assertTrue(n1.rootedEquals(n2));
	n1 = bin(bin(1,2),3);
	((GeneNode)n1.findLeaf(1).getParent()).setCollapsedEdge(true);
	n1.collapseEdges();
	Assert.assertTrue(n1.rootedEquals(tri(1,2,3)));
    }

    public void testEqualsCompareTo()  {
	Assert.assertTrue(!new GeneNode(et,-1).equals(new GeneLeaf(et,1)));
	Assert.assertTrue(!new GeneLeaf(et,1).equals(new GeneNode(et,-1)));
	Assert.assertTrue(new GeneNode(et,-1,1).equals(new GeneNode(et,-1,2)));
	Assert.assertFalse(new GeneNode(et,-1,1).equals(new GeneNode(et,-2,2)));
	Assert.assertTrue(new GeneNode(et,-1,1).compareTo(new GeneNode(et,-1,2))==0);
	Assert.assertTrue(new GeneNode(et,-1,1).compareTo(new GeneNode(et,-2,2))>0);
	Assert.assertTrue(new GeneNode(et,-3,1).compareTo(new GeneNode(et,-2,2))<0);
    }

    public void testCloneTree()  {
	GeneNode r,n0,n1;
	n1 = (GeneNode)GeneNode.nextNode(et).addChild(new GeneLeaf(et,1))
	    .addChild(new GeneLeaf(et,2));
	n0 = (GeneNode)GeneNode.nextNode(et).addChild(new GeneLeaf(et,3))
	    .addChild(new GeneLeaf(et,4));
	r = (GeneNode)GeneNode.nextNode(et).addChild(n1).addChild(n0);
	Assert.assertTrue(r.rootedEquals(r.cloneTree()));
	Assert.assertTrue(r.cloneTree().rootedEquals(r));
	Assert.assertFalse(r.cloneTree().rootedEquals(n1));
	Assert.assertFalse(n0.cloneTree().rootedEquals(r));
    }

    public void testAddRemoveChildHasChildren()  {
	GeneNode n = GeneNode.nextNode(et);
	GeneNode r = (GeneNode)GeneNode.nextNode(et).addChild(new GeneLeaf(et,1))
	    .addChild(new GeneLeaf(et,2)).addChild(n);
	GeneLeaf l = new GeneLeaf(et,100);
	GeneNode n1 = (GeneNode)GeneNode.nextNode(et).addChild(l);
	// removeChildren
	Assert.assertTrue(!r.removeChild(l));
	Assert.assertTrue(l.getParent()!=null);
	Assert.assertFalse(r.removeChild(new GeneLeaf(et,6)));
	Assert.assertTrue(r.numChildren()==3);
	Assert.assertTrue(r.removeChild(n));
	Assert.assertTrue(r.numChildren()==2);
	Assert.assertTrue(r.removeChild(new GeneLeaf(et,1)));
	Assert.assertTrue(r.numChildren()==1);
	Assert.assertTrue(r.removeChild(new GeneLeaf(et,2)));
	Assert.assertTrue(r.numChildren()==0);
	// hasChildren
	Assert.assertFalse(r.hasChildren());
	Assert.assertFalse(n.hasChildren());
	r.addChild(new GeneLeaf(et,3));
	Assert.assertTrue(r.hasChildren());
	GeneLeaf l3 = new GeneLeaf(et,3);
	try {
	    r.addChild(l3);
	    r.addChild(l3);
	    Assert.assertTrue(false);
	} catch(FatalException e) {
	    Assert.assertTrue(l3.getParent()==null);
	}
    }

    public void testPruneOneChildNodes() throws IOException  {
	GeneNode n,r;
	n = GeneNode.nextNode(et);
	r = (GeneNode)GeneNode.nextNode(et).addChild(GeneNode.nextNode(et).addChild(n));
	n.addChild(new GeneLeaf(et,1)).addChild(new GeneLeaf(et,2));
	//System.out.println(r.toStringAll());
	Assert.assertTrue( ((GeneNode)r.cloneTree().pruneOneChildNodes())
			  .rootedEquals(n));
	Assert.assertTrue( ((GeneNode)r.cloneTree().pruneOneChildNodes()
			  .pruneOneChildNodes()).rootedEquals(n));
	Assert.assertFalse(((GeneNode)n.cloneTree().pruneOneChildNodes())
			   .rootedEquals(r));
	// 2002-11-19
	r =(GeneNode)GeneNode.nextNode(et).addChild(bin(new GeneLeaf(et,2),bin(3,4)));
	r = bin(new GeneLeaf(et,1),r);
	n = bin(new GeneLeaf(et,1),bin(new GeneLeaf(et,2),bin(3,4)));
	r = (GeneNode)r.pruneOneChildNodes();
	//	System.out.println("\n" + r.root().toStringAll(null));
	//	System.out.println("\n" + n.root().toStringAll(null));
	Assert.assertTrue(r.rootedEquals(n));
	// 2002-12-09, Ser till att vrdet p kanten hanteras rtt
	// - UNKNOWN -> behll vrdet lngst ner
	r = new GeneLeaf(et,1,10.0);
	r = (GeneNode)GeneNode.nextNode(et,20.0).addChild(r);
	n = bin(2,r);
	n = (GeneNode)n.pruneOneChildNodes();
	Assert.assertTrue(n.findLeaf(1).getEdgeValue()==10.0);
	// - BOOTSTRAP -> behll det minsta vrdet. En kedja r inte starkare
	// n sin svagaste lnk.
	r = new GeneLeaf(etBO,1,10.0);
	r = (GeneNode)GeneNode.nextNode(etBO, 10.0).addChild(r);
	r = (GeneNode)GeneNode.nextNode(etBO, 10.0).addChild(r);
	n = bin(2,r);
	n = (GeneNode)n.pruneOneChildNodes();
	Assert.assertEquals(new Double(n.findLeaf(1).getEdgeValue()),
			    new Double(30.0));
	// - ML -> Avstnd lger man ihop
	r = new GeneLeaf(etML,1,30.0);
	r = (GeneNode)GeneNode.nextNode(etML, 10.0).addChild(r);
	r = (GeneNode)GeneNode.nextNode(etML, 20.0).addChild(r);
	n = bin(2,r);
	n = (GeneNode)n.pruneOneChildNodes();
	Assert.assertTrue(n.findLeaf(1).getEdgeValue()==10.0);
    }

    
    /*   
    public void testFindLeaf()  {
	GeneNode root = mlDNA4091();
	GeneLeaf l3 = root.findLeaf(3);
	GeneLeaf l2 = root.findLeaf(2);
	Assert.assertTrue(l3.label()==3);
	Assert.assertTrue(l2.label()==2);
	Assert.assertTrue(l3.getParent()==l2.getParent());
	Assert.assertTrue(l3.getParent()!=root.findLeaf(4).getParent());
	Assert.assertTrue(l3.getParent().equals(l2.getParent()));
	Assert.assertTrue(mlDNA4091().findLeaf(1000)==null);
    }

    public void testGetLeaves()  {
	GeneNode root = mlDNA4091();
	Set leafs = root.getLeaves();
	Assert.assertTrue(leafs.contains(new GeneLeaf(et,1)));
	Assert.assertTrue(leafs.contains(new GeneLeaf(et,2)));
	Assert.assertTrue(leafs.contains(new GeneLeaf(et,3)));
	Assert.assertTrue(leafs.contains(new GeneLeaf(et,4)));
	Assert.assertTrue(leafs.contains(new GeneLeaf(et,5)));
	Assert.assertTrue(leafs.size()==5);
	root.findLeaf(4).getParent().addChild(new GeneLeaf(et,2));
	try {
	    root.getLeaves();
	    Assert.assertTrue(false);
	} catch(FatalException e) {};

	leafs = new TreeSet();
	leafs.add(new GeneLeaf(et,1));
	Assert.assertTrue(new GeneLeaf(et,1).getLeaves().equals(leafs));
    }
    */

    public void testReroot() throws IOException {
	GeneNode r1,r2;
	GeneNode n1,n2,n3,n4;
	GeneLeaf l1,l2,l3,l4,l5;
	// 2002-12-23, additional mixed test(bootstram and ML).
	l1 = new GeneLeaf(etBO,1,100.0);
	l2 = new GeneLeaf(etML,2,0.2);
	l3 = new GeneLeaf(etML,3,0.3);
	l4 = new GeneLeaf(etBO,4,80.0);
	l5 = new GeneLeaf(etML,5,0.8);
	n4 = bin(l1,l2).setEdgeType(etML).setEdgeValue(0.5);
	n3 = bin(n4,l3).setEdgeType(etBO).setEdgeValue(95.0);
	n2 = bin(n3,l4).setEdgeType(etML).setEdgeValue(0.8);
	n1 = bin(n2,l5).setEdgeType(etUN);
	// leaf 1
	r1 = n1.cloneTree().findLeaf(1).reroot();
	//Assert.assertTrue(r1.getEdgeType().equals(etRO));
	Assert.assertTrue(r1.findLeaf(1).getEdgeValue()==50.0);
	Assert.assertTrue(r1.findLeaf(1).getEdgeType().equals(etBO));
	Assert.assertTrue(r1.findLeaf(2).getEdgeValue()==0.2);
	Assert.assertTrue(r1.findLeaf(2).getEdgeType().equals(etML));
	Assert.assertTrue(r1.findLeaf(3).getEdgeValue()==0.3);
	Assert.assertTrue(r1.findLeaf(3).getEdgeType().equals(etML));
	Assert.assertTrue(r1.findLeaf(4).getEdgeValue()==80.0);
	Assert.assertTrue(r1.findLeaf(4).getEdgeType().equals(etBO));
	Assert.assertTrue(r1.findLeaf(5).getEdgeValue()==0.8);
	Assert.assertTrue(r1.findLeaf(5).getEdgeType().equals(etML));
	Assert.assertTrue(((GeneNode)r1.findLeaf(4).getParent()).getEdgeValue()==95.0);
	Assert.assertTrue(((GeneNode)r1.findLeaf(4).getParent()).getEdgeType()
			  .equals(etBO));
	Assert.assertTrue(((GeneNode)r1.findLeaf(3).getParent()).getEdgeValue()==0.5);
	Assert.assertTrue(((GeneNode)r1.findLeaf(3).getParent()).getEdgeType()
			  .equals(etML));
	Assert.assertTrue(((GeneNode)r1.findLeaf(2).getParent()).getEdgeValue()==50.0);
	Assert.assertTrue(((GeneNode)r1.findLeaf(2).getParent()).getEdgeType()
			  .equals(etBO));
	// leaf 3
	r1 = n1.cloneTree().findLeaf(3).reroot();
	//Assert.assertTrue(r1.getEdgeType().equals(etRO));
	Assert.assertTrue(r1.findLeaf(1).getEdgeValue()==100.0);
	Assert.assertTrue(r1.findLeaf(1).getEdgeType().equals(etBO));
	Assert.assertTrue(r1.findLeaf(2).getEdgeValue()==0.2);
	Assert.assertTrue(r1.findLeaf(2).getEdgeType().equals(etML));
	Assert.assertTrue(r1.findLeaf(3).getEdgeValue()==0.3);
	Assert.assertTrue(r1.findLeaf(3).getEdgeType().equals(etML));
	Assert.assertTrue(r1.findLeaf(4).getEdgeValue()==80.0);
	Assert.assertTrue(r1.findLeaf(4).getEdgeType().equals(etBO));
	Assert.assertTrue(r1.findLeaf(5).getEdgeValue()==0.8);
	Assert.assertTrue(r1.findLeaf(5).getEdgeType().equals(etML));
	Assert.assertTrue(((GeneNode)r1.findLeaf(4).getParent()).getEdgeValue()==95.0);
	Assert.assertTrue(((GeneNode)r1.findLeaf(4).getParent()).getEdgeType()
			  .equals(etBO));
	Assert.assertTrue(((GeneNode)r1.findLeaf(2).getParent()).getEdgeValue()==0.5);
	Assert.assertTrue(((GeneNode)r1.findLeaf(2).getParent()).getEdgeType()
			  .equals(etML));
	Assert.assertTrue(((GeneNode)r1.findLeaf(1).getParent().getParent())
			  .getEdgeValue()==0.3);
	Assert.assertTrue(((GeneNode)r1.findLeaf(1).getParent().getParent())
			  .getEdgeType().equals(etML));
	// leaf 4-3
	r1 = ((GeneNode)n1.cloneTree().findLeaf(3).getParent()).reroot();
	//Assert.assertTrue(r1.getEdgeType().equals(etRO));
	Assert.assertTrue(r1.findLeaf(1).getEdgeValue()==100.0);
	Assert.assertTrue(r1.findLeaf(1).getEdgeType().equals(etBO));
	Assert.assertTrue(r1.findLeaf(2).getEdgeValue()==0.2);
	Assert.assertTrue(r1.findLeaf(2).getEdgeType().equals(etML));
	Assert.assertTrue(r1.findLeaf(3).getEdgeValue()==0.3);
	Assert.assertTrue(r1.findLeaf(3).getEdgeType().equals(etML));
	Assert.assertTrue(r1.findLeaf(4).getEdgeValue()==80.0);
	Assert.assertTrue(r1.findLeaf(4).getEdgeType().equals(etBO));
	Assert.assertTrue(r1.findLeaf(5).getEdgeValue()==0.8);
	Assert.assertTrue(r1.findLeaf(5).getEdgeType().equals(etML));
	Assert.assertTrue(((GeneNode)r1.findLeaf(4).getParent()).getEdgeValue()==47.5);
	Assert.assertTrue(((GeneNode)r1.findLeaf(4).getParent())
			  .getEdgeType().equals(etBO));
	Assert.assertTrue(((GeneNode)r1.findLeaf(2).getParent()).getEdgeValue()==0.5);
	Assert.assertTrue(((GeneNode)r1.findLeaf(2).getParent())
			  .getEdgeType().equals(etML));
	Assert.assertTrue(((GeneNode)r1.findLeaf(3).getParent()).getEdgeValue()==47.5);
	Assert.assertTrue(((GeneNode)r1.findLeaf(3).getParent())
			  .getEdgeType().equals(etBO));
	// 2002-12-18, additional bootstrap test
	l1 = new GeneLeaf(etBO,1,100.0);
	l2 = new GeneLeaf(etBO,2,100.0);
	l3 = new GeneLeaf(etBO,3,100.0);
	l4 = new GeneLeaf(etBO,4,100.0);
	l5 = new GeneLeaf(etBO,5,100.0);
	n4 = bin(l1,l2).setEdgeType(etBO).setEdgeValue(4.0);
	n3 = bin(n4,l3).setEdgeType(etBO).setEdgeValue(3.0);
	n2 = bin(n3,l4).setEdgeType(etBO).setEdgeValue(100.0);
	n1 = bin(n2,l5).setEdgeType(etUN);
	// root by take leaf 1
	r1 = n1.cloneTree().findLeaf(1).reroot();
	Assert.assertTrue(((GeneNode)r1.findLeaf(5).getParent()).getEdgeValue()==3.0);
	Assert.assertTrue(((GeneNode)r1.findLeaf(3).getParent()).getEdgeValue()==4.0);
	Assert.assertTrue(((GeneNode)r1.findLeaf(2).getParent()).getEdgeValue()==50.0);
	for(Iterator i = r1.getNodes().iterator();i.hasNext();) {
	    GeneNode n = (GeneNode)i.next();
	    Assert.assertTrue( n.equals(r1) || n.getEdgeType().equals(etBO) );
	}
	// root by take leaf 3
	r1 = n1.cloneTree().findLeaf(3).reroot();
	Assert.assertTrue(((GeneNode)r1.findLeaf(5).getParent()).getEdgeValue()==3.0);
	Assert.assertTrue(((GeneNode)r1.findLeaf(1).getParent()).getEdgeValue()==4.0);
	Assert.assertTrue(((GeneNode)r1.findLeaf(2).getParent().getParent())
			  .getEdgeValue()==50.0);
	for(Iterator i = r1.getNodes().iterator();i.hasNext();) {
	    GeneNode n = (GeneNode)i.next();
	    Assert.assertTrue( n.equals(r1) || n.getEdgeType().equals(etBO) );
	}
	// root by take leaf 3-4
	r1 = ((GeneNode)n1.cloneTree().findLeaf(3).getParent()).reroot();
	Assert.assertTrue(((GeneNode)r1.findLeaf(5).getParent()).getEdgeValue()==1.5);
	Assert.assertTrue(((GeneNode)r1.findLeaf(3).getParent()).getEdgeValue()==1.5);
	Assert.assertTrue(((GeneNode)r1.findLeaf(1).getParent()).getEdgeValue()==4.0);
	for(Iterator i = r1.getLeaves().iterator();i.hasNext();) {
	    Assert.assertTrue( ((GeneNode)i.next()).getEdgeValue()==100.0 );
	}
	for(Iterator i = r1.getNodes().iterator();i.hasNext();) {
	    GeneNode n = (GeneNode)i.next();
	    Assert.assertTrue( n.equals(r1) || n.getEdgeType().equals(etBO) );
	}
	// root by take leaf 2-3
	r1 = ((GeneNode)n1.cloneTree().findLeaf(2).getParent()).reroot();
	Assert.assertTrue(((GeneNode)r1.findLeaf(1).getParent()).getEdgeValue()==2.0);
	Assert.assertTrue(((GeneNode)r1.findLeaf(3).getParent()).getEdgeValue()==2.0);
	Assert.assertTrue(((GeneNode)r1.findLeaf(4).getParent()).getEdgeValue()==3.0);
	for(Iterator i = r1.getLeaves().iterator();i.hasNext();) {
	    Assert.assertTrue( ((GeneNode)i.next()).getEdgeValue()==100.0 );
	}
	for(Iterator i = r1.getNodes().iterator();i.hasNext();) {
	    GeneNode n = (GeneNode)i.next();
	    Assert.assertTrue( n.equals(r1) || n.getEdgeType().equals(etBO) );
	}

	/*
	// from start
	r1 = mlDNA4091().findLeaf(1).reroot();
	r2 = bin(bin(bin(4,5),bin(2,3)), new GeneLeaf(et,1));
	Assert.assertTrue(r1.rootedEquals(r2));
	r1 = r1.findLeaf(1).reroot();
	r1 = r1.findLeaf(2).reroot();
	r1 = r1.findLeaf(3).reroot();
	r1 = r1.findLeaf(4).reroot();
	r1 = r1.findLeaf(5).reroot();
	r1 = ((GeneNode)r1.findLeaf(2).getParent()).reroot();
	r2 = bin(bin(bin(4,5),new GeneLeaf(et,1)),bin(2,3));
	Assert.assertTrue(r1.rootedEquals(r2));
	// 2002-11-19
	r1 = bin(new GeneLeaf(et,1),bin(new GeneLeaf(et,2),bin(3,4)));
	r2 = r1.cloneTree().findLeaf(1).reroot();
	Assert.assertTrue(r1.rootedEquals(r2));
	// 2002-12-09, Nr kanten delas s mste den delas p rtt stt
	GeneNode n5,n6;
	// - UNKNOWN, finns inga krav
	// - BOOTSTRAP -> behller samma vrde. En kedja r inte starkare
	//                n sin svagaste lnk.
	l1 = new GeneLeaf(etBO,1,10.0);
	l2 = new GeneLeaf(etBO,2,10.0);
	r1 = bin(l1,l2);
	r2 = r1.cloneTree().findLeaf(1).reroot();
	Assert.assertTrue(r2.findLeaf(1).getEdgeValue()==10.0);
	Assert.assertTrue(r2.findLeaf(2).getEdgeValue()==10.0);
	r2 = r1.cloneTree().findLeaf(2).reroot();
	Assert.assertTrue(r2.findLeaf(1).getEdgeValue()==10.0);
	Assert.assertTrue(r2.findLeaf(2).getEdgeValue()==10.0);
	// - ML -> Avstnd lger man ihop eller delar
	l1 = new GeneLeaf(etML,1,11.0);
	l2 = new GeneLeaf(etML,2,21.0);
	r1 = bin(l1,l2);
	r2 = r1.cloneTree().findLeaf(1).reroot();
	Assert.assertTrue(r2.findLeaf(1).getEdgeValue()==5.5);
	Assert.assertTrue(r2.findLeaf(2).getEdgeValue()==26.5);
	r2 = r1.cloneTree().findLeaf(2).reroot();
	Assert.assertTrue(r2.findLeaf(1).getEdgeValue()==21.5);
	Assert.assertTrue(r2.findLeaf(2).getEdgeValue()==10.5);
	// 2002-12-09, reroot() p rooten ger samma trd
	r1 = bin(1,2);
	Assert.assertTrue(r1.label()==r1.reroot().label());
	*/
    }
    

    /*
    public void testBinaryTree()  {
	GeneNode r1,r2;
	r1 = bin(bin(1,2),(GeneNode)bin(3,4).addChild(new GeneLeaf(et,5)));
	Assert.assertTrue(!r1.binaryTree());
	r1.findLeaf(5).getParent().removeChild(new GeneLeaf(et,5));
	Assert.assertTrue(r1.binaryTree());
	Assert.assertTrue(!mlDNA4091().binaryTree());
	//System.out.println(mlDNA4091().toStringAll());
	Assert.assertTrue(mlDNA4091().findLeaf(1).reroot().binaryTree());
    }
    */
    
    /*
    public void testRerootThis()  {
	GeneNode rr = ((GeneNode)mlDNA4091().findLeaf(3).getParent()).rerootThis();
	GeneNode tn = (GeneNode)GeneNode.nextNode(et,1).addChild(new GeneLeaf(et,4,4))
	    .addChild(new GeneLeaf(et,5,5));
	tn = (GeneNode)GeneNode.nextNode(et,1).addChild(tn).addChild(new GeneLeaf(et,1,1));
	tn = (GeneNode)GeneNode.nextNode(et,1).addChild(tn).addChild(new GeneLeaf(et,3,3))
	    .addChild(new GeneLeaf(et,2,2));
	Assert.assertTrue(tn.rootedEquals(rr) && rr.rootedEquals(tn));
	Assert.assertTrue(rr.getParent()==null);
	rr = ((GeneNode)rr.findLeaf(5).getParent()).rerootThis();
	tn = ((GeneNode)mlDNA4091().findLeaf(4).getParent()).rerootThis();
	Assert.assertTrue(tn.rootedEquals(rr) && rr.rootedEquals(tn));
	Assert.assertTrue(rr.getParent()==null && tn.getParent()==null);
	tn = ((GeneNode)mlDNA4091().findLeaf(3).getParent()).rerootThis();
	Assert.assertFalse(tn.rootedEquals(rr) && rr.rootedEquals(tn));
	rr = ((GeneNode)distDNA993().findLeaf(3).getParent()).rerootThis();
	tn = (GeneNode)GeneNode.nextNode(et,1).addChild(new GeneLeaf(et,4,4)).
	    addChild(new GeneLeaf(et,2,2));
	tn = (GeneNode)GeneNode.nextNode(et,1).addChild(tn).addChild(new GeneLeaf(et,1,1));
	tn = (GeneNode)GeneNode.nextNode(et,1).addChild(tn).addChild(new GeneLeaf(et,5,5))
	    .addChild(new GeneLeaf(et,3,3));
	Assert.assertTrue(tn.rootedEquals(rr) && rr.rootedEquals(tn));
	rr = ((GeneNode)rr.findLeaf(1).getParent()).rerootThis();
	tn = ((GeneNode)distDNA993().findLeaf(1).getParent()).rerootThis();
	Assert.assertTrue(tn.rootedEquals(rr) && rr.rootedEquals(tn));
	tn = ((GeneNode)distDNA993().findLeaf(4).getParent()).rerootThis();
	Assert.assertFalse(tn.rootedEquals(rr) && rr.rootedEquals(tn));
    }
    */
    
    public void testRootedEquals()  {
	GeneNode n1,n2;
	n1 = (GeneNode)GeneNode.nextNode(et,1).addChild(new GeneLeaf(et,3,3)).
	    addChild(new GeneLeaf(et,2,2));
	n2 = (GeneNode)GeneNode.nextNode(et,1).addChild(new GeneLeaf(et,4,4)).
	    addChild(new GeneLeaf(et,5,5));
	GeneNode r1 = (GeneNode)GeneNode.nextNode(et,1).addChild(n2).addChild(n1);
	r1.addChild(new GeneLeaf(et,1,1));
	
	GeneNode r2 = (GeneNode)GeneNode.nextNode(et,1).addChild(new GeneLeaf(et,1,1));
	n1 = (GeneNode)GeneNode.nextNode(et,1).addChild(new GeneLeaf(et,2,2)).
	    addChild(new GeneLeaf(et,3,3));
	n2 = (GeneNode)GeneNode.nextNode(et,1).addChild(new GeneLeaf(et,5,5)).
	    addChild(new GeneLeaf(et,4,4));
	r2.addChild(n2).addChild(n1);
	
	Assert.assertTrue(r1.rootedEquals(r2));
	Assert.assertTrue(r2.rootedEquals(r1));
	
	GeneNode r3 = (GeneNode)GeneNode.nextNode(et,1).addChild(new GeneLeaf(et,2,2));
	n1 = (GeneNode)GeneNode.nextNode(et,1).addChild(new GeneLeaf(et,3,3)).
	    addChild(new GeneLeaf(et,5,5));
	n2 = (GeneNode)GeneNode.nextNode(et,1).addChild(new GeneLeaf(et,4,4)).
	    addChild(new GeneLeaf(et,1,1));
	r3.addChild(n2).addChild(n1);
	
	Assert.assertFalse(r3.rootedEquals(r2));
	Assert.assertFalse(r3.rootedEquals(r1));
	Assert.assertFalse(r1.rootedEquals(r3));
	Assert.assertFalse(r2.rootedEquals(r3));
	
	n2 = (GeneNode)GeneNode.nextNode(et,1).addChild(new GeneLeaf(et,4,4))
	    .addChild(new GeneLeaf(et,3,3));
	GeneNode r4 = (GeneNode)GeneNode.nextNode(et,1).addChild(n2);
	r4.addChild(new GeneLeaf(et,1,1));

	Assert.assertFalse(r3.rootedEquals(r1));
	Assert.assertFalse(r3.rootedEquals(r2));
	Assert.assertTrue(r3.rootedEquals(r3));
	Assert.assertFalse(r3.rootedEquals(r4));
	Assert.assertFalse(r1.rootedEquals(r4));
	Assert.assertFalse(r2.rootedEquals(r4));
	Assert.assertFalse(r3.rootedEquals(r4));
	Assert.assertTrue(r4.rootedEquals(r4));
    }
    
    public void testHasChillrenSumChildren()  {
	GeneNode n = GeneNode.nextNode(et);
	Assert.assertTrue(!n.hasChildren());
	GeneLeaf l1 = new GeneLeaf(et,1);
	GeneLeaf l2 = new GeneLeaf(et,2);
	n.addChild(l1);
	Assert.assertTrue(n.hasChildren());
	Assert.assertTrue(n.numChildren()==1);
	n.addChild(l2);
	Assert.assertTrue(n.hasChildren());
	Assert.assertTrue(n.numChildren()==2);
	n.removeChild(l1);
	n.removeChild(l2);
	Assert.assertTrue(!n.hasChildren());
	Assert.assertTrue(n.numChildren()==0);
    }
    
    public void testSetGet()   {
	GeneNode p = GeneNode.nextNode(et);
	GeneNode n = (GeneNode)new GeneNode(et,-10,1);
	p.addChild(n);
	Assert.assertTrue(n.getEdgeValue()==(double)1);
	n.setEdgeValue(2);
	Assert.assertTrue(n.getEdgeValue()==(double)2);
	Assert.assertTrue(n.getLabel()==-10);
	//n.setLabel(-20);
	//Assert.assertTrue(n.getLabel()==-20);
    }
    
}
