/*
 *  TestSplits.java
 *
 *
 */

package softparsmap.tests;
import java.util.*;
import java.io.*;
import softparsmap.*;
import junit.framework.*;

public class TestSplits extends CommonTestCase {

    public static void main(String[] args) {
	junit.textui.TestRunner.run(suite());
    }

    public static Test suite() {
	return new TestSuite(TestSplits.class);
    }

    public void testSplits() {
	Split d1 = newSplit("1", "2");
	Split d2 = newSplit("1,2", "3");
	Split d3 = newSplit("1,2", "3,4");
	Split d4 = newSplit("1,3", "2,4");
	
	Assert.assertTrue(d1.isConsistentWith(d1));
	Assert.assertTrue(d1.isConsistentWith(d2));
	Assert.assertTrue(d1.isConsistentWith(d3));
	Assert.assertTrue(d1.isConsistentWith(d4));

	Assert.assertTrue(!d2.isConsistentWith(d1));
	Assert.assertTrue(d2.isConsistentWith(d2));
	Assert.assertTrue(d2.isConsistentWith(d3));
	Assert.assertTrue(d2.isConsistentWith(d4));

	Assert.assertTrue(!d3.isConsistentWith(d1));
	Assert.assertTrue(d3.isConsistentWith(d2));
	Assert.assertTrue(d3.isConsistentWith(d3));
	Assert.assertTrue(!d3.isConsistentWith(d4));

	Assert.assertTrue(!d4.isConsistentWith(d1));
	Assert.assertTrue(!d4.isConsistentWith(d2));
	Assert.assertTrue(!d4.isConsistentWith(d3));
	Assert.assertTrue(d4.isConsistentWith(d4));
    }
    public Split newSplit(String l1, String l2) {
	return new Split(getSet(l1), getSet(l2));	
    }
    public Set getSet(String l1) {
	Set sl1 = new TreeSet();
	StringTokenizer st = new StringTokenizer(l1, ",");
	while (st.hasMoreTokens()) {
	    sl1.add(new Integer(st.nextToken().trim()));
	}
	return sl1;
    }
    

    public void testConsistency() {
	GeneNode g1, g2, g3, g4;

	//
	g1 = parseNewickNe(et, "((((1,3),2),(4,5)),6)");
	g2 = parseNewickNe(et, "((((1,2),3),(4,5)),(6,7))");
	Assert.assertTrue(!g1.isConsistentWith(g2));
	Assert.assertTrue(!g2.isConsistentWith(g1));
	
	//
	g1 = parseNewickNe(et, "((((1,2),3),4),5)");
	g2 = parseNewickNe(et, "(1,(2,3))");
	g3 = parseNewickNe(et, "((1,2),(3,4))");
	g4 = parseNewickNe(et, "((1,3),(2,4))");

	Assert.assertTrue(g1.isConsistentWith(g1));
	Assert.assertTrue(!g1.isConsistentWith(g2));
	Assert.assertTrue(!g1.isConsistentWith(g3));
	Assert.assertTrue(!g1.isConsistentWith(g4));

	Assert.assertTrue(g2.isConsistentWith(g1));
	Assert.assertTrue(g2.isConsistentWith(g2));
	Assert.assertTrue(g2.isConsistentWith(g3));
	Assert.assertTrue(g2.isConsistentWith(g4));
	
	Assert.assertTrue(g3.isConsistentWith(g1));
	Assert.assertTrue(!g3.isConsistentWith(g2));
	Assert.assertTrue(g3.isConsistentWith(g3));
	Assert.assertTrue(!g3.isConsistentWith(g4));
	
	Assert.assertTrue(!g4.isConsistentWith(g1));
	Assert.assertTrue(!g4.isConsistentWith(g2));
	Assert.assertTrue(!g4.isConsistentWith(g3));
	Assert.assertTrue(g4.isConsistentWith(g4));



	
    }



}
