/*
 *  TestNodeLeaf.java  ;  ver 1.0  ; 2002-10-08
 *
 *
 */

package softparsmap.tests;
import java.util.*;
import java.io.*;
import softparsmap.*;
import junit.framework.*;

public class TestTreeParser extends CommonTestCase {

    public static void main(String[] args) {
	junit.textui.TestRunner.run(suite());
    }

    public static Test suite() {
	return new TestSuite(TestTreeParser.class);
    }

    public void testStringTemplate() {
	StringTemplate st;

	// added 2006-04-27
	st = new StringTemplate("family{number}.tree","{","}");
	try {
	    st.paintTemplate("map5646001.tree");
	    Assert.assertTrue(false);
	} catch (Exception e) {}

	//
	st = new StringTemplate("","{","}");
	Assert.assertEquals("", st.getPaintedTemplate());
	
	//
	st = new StringTemplate("{1}","{","}");
	st.setMarker("{1}", "a");
	Assert.assertEquals("a", st.getPaintedTemplate());
	
	//
	st = new StringTemplate("{1}:{2}","{","}");
	st.setMarker("{1}", "a");
	st.setMarker("{2}", "b");
	Assert.assertEquals("a:b", st.getPaintedTemplate());
	
	//
	st = new StringTemplate(":{1}:{2}:","{","}");
	st.setMarker("{1}", "a");
	st.setMarker("{2}", "b");
	Assert.assertEquals(":a:b:", st.getPaintedTemplate());
	
	//
	st = new StringTemplate("no_marker","{","}");
	st.paintTemplate("no_marker");
	Assert.assertTrue(!st.hasMarker("{}"));
	
	//
	st = new StringTemplate("","{","}");
	st.paintTemplate("");
	Assert.assertTrue(!st.hasMarker("{}"));
		
	//
	st = new StringTemplate(":{1}","{","}");
	st.paintTemplate(":a");
	Assert.assertEquals(st.getFirstValue("{1}"), "a");

	//
	st = new StringTemplate("{1}:","{","}");
	st.paintTemplate("a:");
	Assert.assertEquals(st.getFirstValue("{1}"), "a");
	
	//
	st = new StringTemplate("{1}","{","}");
	st.paintTemplate("a");
	Assert.assertEquals(st.getFirstValue("{1}"), "a");
	
	//
	st = new StringTemplate(":{1}:","{","}");
	st.paintTemplate(":a:");
	Assert.assertEquals(st.getFirstValue("{1}"), "a");
	
	//
	st = new StringTemplate(":{1}:{2}:","{","}");
	st.paintTemplate(":a:b:");
	Assert.assertEquals(st.getFirstValue("{1}"), "a");
	Assert.assertEquals(st.getFirstValue("{2}"), "b");

	//
	st = new StringTemplate("{1}:{2}","{","}");
	st.paintTemplate("a:b");
	Assert.assertEquals(st.getFirstValue("{1}"), "a");
	Assert.assertEquals(st.getFirstValue("{2}"), "b");
	
    }
    
    public void testStandardParser() {
	GeneNode n = null;
	AbstractTreeParser tp = null;

	//
	tp = (AbstractTreeParser)
	    pf.getTag("tree_parser::newick").getTagInstance();
	testStandardParserSub(tp, "(1,(2,3));", 5);
	testStandardParserSub(tp, "(1,(2,3,4),(5,(6,7)),8);", 12);
	testStandardParserSub(tp, "(1,2);", 3);

	// testing one leaf
	tp = (AbstractTreeParser)
	    pf.getTag("tree_parser::newick_edge_value").getTagInstance();
	testStandardParserSub(tp, "(1);", 1);
	Assert.assertEquals(tp.parseOut(new GeneLeaf(null, 1)), "(1);");
	
	// 
	tp = (AbstractTreeParser)pf.getTag
	    ("tree_parser::newick_edge_value_leaves_too").getTagInstance();
	testStandardParserSub(tp, "(1:100.0,2:100.0);", 3);
	
    }
    public void testStandardParserSub(AbstractTreeParser tp, String tree,
				      int nodes) {
	// testing parsing in
	GeneNode n = (GeneNode)tp.parseIn(tree);
	Assert.assertTrue(n.isRoot());
	Assert.assertTrue(n.getNodes().size()==nodes);
	// parsing out
	String out = tp.parseOut(n);
	//System.out.println("*** tree="+tree+"  |  out="+out);
	Assert.assertEquals(tree, tp.parseOut(n));
	
    }
    

}
