
package softparsmap.tests;
import java.util.*;
import java.io.*;
import softparsmap.*;
import junit.framework.*;

public class TestTreeParserDual extends CommonTestCase {

    public static void main(String[] args) {
	junit.textui.TestRunner.run(suite());
    }

    public static Test suite() {
	return new TestSuite(TestTreeParserDual.class);
    }

    public void test20051115() {
	Node tree;
	TreeParser tpIn = (TreeParser)
	    pf.getTag("tree_parser::schreiber_gene_label").getTagInstance();
	TreeParser tpMap = (TreeParser)pf.getTag
	    ("tree_parser::labels").getTagInstance();

	// 2005-11-18
	tree = tpIn.parseIn("[[1,2],[3,[1]]]");
	tpMap.parseIn(tree, "[[1,2],[3,[1]]]");
	Assert.assertEquals("2", tree.getMap().get("label"));

	//System.out.println("### "+tree.toStringAll());

	//
	String input = "n := [[8, 5], [1, 14], [12, [1]], [4, [2]], [13, [4]], [[3], [5]]]:";
	tree = tpIn.parseIn(input);
	tpMap.parseIn(tree, input);



    }

    public void testSchreiberInfoGene() {
	TreeParser tpIn = (TreeParser)
	    pf.getTag("tree_parser::schreiber_gene_label").getTagInstance();
	TreeParser tpMapIn = (TreeParser)pf.getTag
	    ("tree_parser::schreiber_info_gene_internal_label")
	    .getTagInstance();

	TreeParser tpMap = (TreeParser)pf.getTag
	    ("tree_parser::schreiber_info_gene_label")
	    .getTagInstance();

	// testing adding info using schreiber structure parser
	mapInfoInOut(tpIn, "[[1]]", tpMap, "[[1]]");
	mapInfoInOut(tpIn, "[[1,2],[[1],3]]", tpMap, "[[1,2],[[1],3]]");
	mapInfoInOut(tpIn, "[[1,2]]", tpMap, "[[1,2]]");
	mapInfoInOut(tpIn, "[[1,2],[[1],3],[4,5],[[2],[3]]]", 
		     tpMap, "[[1,2],[[1],3],[4,5],[[2],[3]]]");
	
	// testing adding info using schreiber internal node structure parser
	mapInfoInOut(tpIn, "[[1,2],[[1],3]]", tpMapIn, "[[1],[2]]");
	mapInfoInOut(tpIn, "[[1,2]]", tpMapIn, "[[1]]");
	mapInfoInOut(tpIn, "[[1]]", tpMapIn, "[]");
	mapInfoInOut(tpIn, "[[1,2],[[1],3],[4,5],[[2],[3]]]", 
		     tpMapIn, "[[1],[2],[3],[4]]");
	
    }

    public void testSchreiberGeneLabel() {
	TreeParser tp = (TreeParser)
	    pf.getTag("tree_parser::schreiber_gene_label").getTagInstance();
	Node tree = null;

	// testing in and out parsers
	circle(tp, "[[1]]");
	circle(tp, "n:=[[1,2],[[1],3]]:", "[[1,2],[[1],3]]");
	circle(tp, " [ [ 1 , 2 ] , [[ 1 ],3] ]", "[[1,2],[[1],3]]");
	
	circle(tp, "[[1,2],[[1],3],[4,5],[[2],[3]]]");
	circle(tp, "[[1,2],[[1],3]]");
	circle(tp, "[[1,2]]");

	// single node
	tree = tp.parseIn("[[1]]");
	Assert.assertTrue(tree.getNodes().size()==1);
    }

    public void mapInfoInOut(TreeParser tp, String strTree, 
			     TreeParser tpInternal, String treeInfo) {
	Node tree = tp.parseIn(strTree);
	tpInternal.parseIn(tree, treeInfo);
	Assert.assertEquals(treeInfo, tpInternal.parseOut(tree));
    }
  
    public void circle(TreeParser tp, String tree) {
	circle(tp, tree, tree);
    }

    public void circle(TreeParser tp, String tree, String after) {
	String after2 = tp.parseOut(tp.parseIn(tree));
	Assert.assertEquals(after, after2);
    }

}
